import unittest

from ginsfsm.smachine import (
    SMachine,
    EMPTY_FSM,
    StateError,
    MachineError,
    EventError,
    EventNotAcceptedError,
    filter_event_attrs,
    )


class TestSuperEmptySMachine(unittest.TestCase):
    def setUp(self):
        self.fsm = SMachine({})

    def test_set_bad_smachine(self):
        self.assertRaises(MachineError, SMachine, '')

    def test_set_bad_new_state(self):
        self.assertRaises(StateError, self.fsm.set_new_state, 'STATE')
        self.assertRaises(StateError, self.fsm._set_new_state, 'STATE')
        self.assertRaises(StateError, self.fsm.set_new_state, 1)
        self.assertRaises(StateError, self.fsm._set_new_state, 1)
        self.assertRaises(StateError, self.fsm.set_new_state, {})
        self.assertRaises(StateError, self.fsm._set_new_state, {})

    def test_get_current_bad_state_name(self):
        st = self.fsm.get_current_state()
        self.assertEqual(st, None)

    def test_trace_mach(self):
        self.fsm.trace_mach = True
        self.fsm.trace_mach = False


class TestEmptySMachine(unittest.TestCase):
    def setUp(self):
        self.fsm = SMachine(EMPTY_FSM)

    def test_set_bad_new_state(self):
        self.assertRaises(StateError, self.fsm.set_new_state, 'STATE')
        self.assertRaises(StateError, self.fsm._set_new_state, 'STATE')
        self.assertRaises(StateError, self.fsm.set_new_state, 1)
        self.assertRaises(StateError, self.fsm._set_new_state, 1)
        self.assertRaises(StateError, self.fsm.set_new_state, {})
        self.assertRaises(StateError, self.fsm._set_new_state, {})

    def test_get_current_bad_state_name(self):
        st = self.fsm.get_current_state()
        self.assertEqual(st, None)


#########################################################3
#   Wrong FSM: state name is not in state_list
#########################################################3
FSM_TEST1 = {
    'event_list': (
        'EV_TIMEOUT:bottom.input',
    ),
    'state_list': (
        'ST_STATE1',
        'ST_STATEXX',
    ),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT', None, 'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT', None, 'ST_STATE1'),
        ),
    }
}


class Test1SMachineState(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST1)

#########################################################3
#   Wrong FSM: state_list has more states than machine
#########################################################3
FSM_TEST2 = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_STATE1', 'ST_STATE2', 'ST_STATE3'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT',      None,       'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT',      None,       'ST_STATE1'),
        ),
    }
}

class Test2SMachineState(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST2)

#########################################################3
#   Wrong FSM: state_list has dup names
#########################################################3
FSM_TEST3 = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_STATE1', 'ST_STATE2', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT',      None,       'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT',      None,       'ST_STATE1'),
        ),
    }
}


class Test3SMachineState(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST3)

#########################################################3
#   Wrong FSM: event name is not in event_list
#########################################################3
FSM_TEST4 = {
    'event_list': ('EV_TIMEOUT1','EV_TIMEOUT2'),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUTXXX',    None,       'ST_STATE2'),
            ('EV_TIMEOUT2',      None,       'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT1',      None,       'ST_STATE1'),
            ('EV_TIMEOUT2',      None,       'ST_STATE1'),
        ),
    }
}


class Test4SMachineEvent(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST4)

#########################################################3
#   Wrong FSM: event name has more names
#########################################################3
FSM_TEST5 = {
    'event_list': ('EV_TIMEOUT1','EV_TIMEOUT2','EV_TIMEOUT3'),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT1',      None,       'ST_STATE2'),
            ('EV_TIMEOUT2',      None,       'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT1',      None,       'ST_STATE1'),
            ('EV_TIMEOUT2',      None,       'ST_STATE1'),
        ),
    }
}


class Test5SMachineEvent(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST5)

#########################################################3
#   Wrong FSM: event name has dup names
#########################################################3
FSM_TEST6 = {
    'event_list': ('EV_TIMEOUT1','EV_TIMEOUT2','EV_TIMEOUT2'),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT1',      None,       'ST_STATE2'),
            ('EV_TIMEOUT2',      None,       'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT1',      None,       'ST_STATE1'),
            ('EV_TIMEOUT2',      None,       'ST_STATE1'),
        ),
    }
}


class Test6SMachineEvent(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST6)

#########################################################3
#   Wrong FSM: next state name is not in state_list
#########################################################3
FSM_TEST7 = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT',      None,       None),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT',      None,       'ST_STATE3'),
        ),
    }
}


class Test7SMachineState(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST7)

#########################################################3
#   Wrong FSM: action is not callable
#########################################################3
FSM_TEST8 = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TIMEOUT',      None,    None),
        ),
        'ST_STATE2':
        (
            ('EV_TIMEOUT',      'hi',       'ST_STATE2'),
        ),
    }
}


class Test8SMachineState(unittest.TestCase):
    def test(self):
        self.assertRaises(MachineError, SMachine, FSM_TEST8)


#########################################################3
#   Good FSM:
#########################################################3
def ac_test1(self, event):
    return 'OK'


def ac_test2(self, event):
    self.set_new_state('ST_STATE1')

FSM_TEST9 = {
    'event_list': (
        'EV_TEST1',
        'EV_TEST2',
        'EV_TEST3:output',
    ),
    'state_list': ('ST_STATE1', 'ST_STATE2'),
    'machine': {
        'ST_STATE1':
        (
            ('EV_TEST1',        None,           'ST_STATE2'),
        ),
        'ST_STATE2':
        (
            ('EV_TEST1',        ac_test1,       'ST_STATE2'),
            ('EV_TEST2',        ac_test2,       None),
        ),
    }
}


class Test9SMachineState(unittest.TestCase):
    def setUp(self):
        self.fsm = SMachine(FSM_TEST9)

    def test_event_list(self):
        self.assertEqual(self.fsm._event_index.get('EV_TEST1'), 1)
        self.assertEqual(self.fsm._event_index.get('EV_TEST2'), 2)
        input_event_list = self.fsm.get_event_list()
        self.assertEqual(input_event_list,
                         filter_event_attrs(FSM_TEST9['event_list']))
        output_event_list = self.fsm.get_output_event_list()
        self.assertEqual(output_event_list, ['EV_TEST3'])

    def test_state_list(self):
        self.assertEqual(self.fsm._state_index.get('ST_STATE1'), 1)
        self.assertEqual(self.fsm._state_index.get('ST_STATE2'), 2)

    def test_inject_event1(self):
        self.assertRaises(EventError, self.fsm.inject_event, 'EV_XXX')
        self.assertRaises(EventError, self.fsm.inject_event, {})
        ret = self.fsm.inject_event('EV_TEST2')
        self.assertEqual(ret, EventNotAcceptedError)
        ret = self.fsm.inject_event('EV_TEST1')
        state = self.fsm.get_current_state()
        self.assertEqual(state, 'ST_STATE2')
        self.assertEqual(ret, None)

        ret = self.fsm.inject_event('EV_TEST1')
        state = self.fsm.get_current_state()
        self.assertEqual(state, 'ST_STATE2')
        self.assertEqual(ret, 'OK')

        ret = self.fsm.inject_event('EV_TEST2')
        state = self.fsm.get_current_state()
        self.assertEqual(state, 'ST_STATE1')
        self.assertEqual(ret, None)



import os
from ginsfsm.compat import configparser
from logging.config import fileConfig


def logging_file_config(config_file, fileConfig=fileConfig,
                        configparser=configparser):
    """
    Setup logging via the logging module's fileConfig function with the
    specified ``config_file``, if applicable.

    ConfigParser defaults are specified for the special ``__file__``
    and ``here`` variables, similar to PasteDeploy config loading.
    """
    parser = configparser.ConfigParser()
    parser.read([config_file])
    if parser.has_section('loggers'):
        config_file = os.path.abspath(config_file)
        return fileConfig(
            config_file,
            dict(__file__=config_file, here=os.path.dirname(config_file))
            )



# -*- encoding: utf-8 -*-
""" GStdin GObj

.. autoclass:: GStdin
    :members: start_up

"""

import sys
import fcntl
import os
from ginsfsm.gobj import GObj
from ginsfsm.c_sock import GSock


def ac_rx_data(self, event):
    data = event.kw.get('data', None)
    for c in data:
        if c == self.config.eol:
            line = ''.join(self.buffer)
            self.buffer = []
            self.broadcast_event('EV_RX_DATA', data=line)
        else:
            self.buffer.append(c)


GSTDIN_FSM = {
    'event_list': (
        'EV_RX_DATA:bottom input:top output',
    ),
    'state_list': (
        'ST_IDLE',
    ),
    'machine': {
        'ST_IDLE':
        (
            ('EV_RX_DATA',      ac_rx_data,     'ST_IDLE'),
        ),
    }
}

GSTDIN_GCONFIG = {
    'eol': [
        str, '\n', 0, None,
        "Caracter defining end of line. LF default"
    ],
    'subscriber': [
        None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
    ],
}


class GStdin(GObj):
    """  GStdin GObj.
    Asynchronous stdin. Use stdin as gsock.

    .. ginsfsm::
       :fsm: GSTDIN_FSM
       :gconfig: GSTDIN_GCONFIG

    *Input-Events:*

        * :attr:`'EV_RX_DATA'`: Receiving data.
            Receiving data from stdin.
            Save the data,
            and broadcast when the line is complete (EOL received).

    *Output-Events:*


    """
    def __init__(self):
        GObj.__init__(self, GSTDIN_FSM, GSTDIN_GCONFIG)
        self.buffer = []

    def start_up(self):
        """ Initialization zone.
        """
        if self.config.subscriber is None:
            self.config.subscriber = self.parent
        self.subscribe_event(None, self.config.subscriber)

        gsock = self.gsock = self.create_gobj(
            None,
            GSock,
            self,
            transmit_ready_event_name=None,
            connected_event_name=None,
            disconnected_event_name=None,
        )

        # make stdin a non-blocking file
        fd = sys.stdin.fileno()
        fl = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

        gsock.set_clisrv_socket(sys.stdin)



"""
WSGI interface.

The wsgi/http server is inspired in `Tornado <http://www.tornadoweb.org/>`_
and `Waitress <http://docs.pylonsproject.org/projects/waitress>`_ projects
and using the best of both.
"""



# -*- coding: utf-8 -*-
"""
    JSONP transport implementation.
"""
import logging

from pyramid.view import view_config
from pyramid.httpexceptions import HTTPNotFound, HTTPServerError

from ginsfsm.gobj import GObj
from ginsfsm.gconfig import GConfig
from ginsfsm.compat import url_unquote_plus
from ginsfsm.protocols.sockjs.server import proto
from ginsfsm.protocols.sockjs.server.basehandler import SessionHandler
from ginsfsm.protocols.sockjs.server.util import bytes_to_str

#----------------------------------------------------------------#
#                   GJsonpSend GClass
#                   /jsonp_send
#----------------------------------------------------------------#
GJSONPSEND_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GJsonpSend(GObj):
    """  GXhr GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GJSONPSEND_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GJsonpSend Views
#                   /jsonp_send
#----------------------------------------------------------------#
@view_config(
    context=GJsonpSend,
    name='',
    attr='options',
    request_method='OPTIONS',
)
@view_config(
    context=GJsonpSend,
    name='',
    attr='post',
    request_method='POST',
)
class JsonpSendHandler(SessionHandler):
    def post(self):
        response = self.response

        # Start response
        self.preflight()
        self.handle_session_cookie()
        self.disable_cache()

        session_id = self.sid = self.context.parent.re_matched_name
        session = self.context.sockjs_server.get_session(session_id)
        if session is None:
            return HTTPNotFound('Session not found')

        data = self.request.body_file.read()  # decode? v1.0.0
        data = bytes_to_str(data)
        ctype = self.request.headers.get('Content-Type', '').lower()
        if ctype == 'application/x-www-form-urlencoded':
            if not data.startswith('d='):
                return HTTPServerError("Payload expected.")

            data = url_unquote_plus(data[2:])

        if not data:
            logging.error('ERROR jsonp_send: Payload expected.')
            return HTTPServerError("Payload expected.")

        try:
            messages = proto.json_decode(data)
        except Exception:
            logging.exception('ERROR jsonp_send: Invalid json encoding')
            return HTTPServerError("Broken JSON encoding.")

        try:
            session.on_messages(messages)
        except Exception:
            logging.exception('ERROR jsonp_send: on_message() failed')
            return HTTPServerError('ERROR jsonp_send: on_message() failed')

        self.set_status(200)
        self.response.content_type = 'text/plain'
        self.response.charset = 'UTF-8'
        self.set_body('ok')
        return response


#----------------------------------------------------------------#
#                   GJsonpPolling GClass
#                   /jsonp
#----------------------------------------------------------------#
GJSONPPOLLING_GCONFIG = {
    'sockjs_server': [None, None, GConfig.FLAG_DIRECT_ATTR, None, ""],
}


class GJsonpPolling(GObj):
    """  GJsonpSend GObj.
    """
    def __init__(self):
        GObj.__init__(self, {}, GJSONPPOLLING_GCONFIG)

    def start_up(self):
        """ Initialization zone.
        """


#----------------------------------------------------------------#
#                   GJsonpPolling Views
#                   /jsonp
#----------------------------------------------------------------#
@view_config(
    context=GJsonpPolling,
    name='',
    attr='options',
    request_method='OPTIONS',
)
@view_config(
    context=GJsonpPolling,
    name='',
    attr='get',
    request_method='GET',
)
class JsonpPollingTransport(SessionHandler):
    """xhr-polling transport implementation"""
    name = 'jsonp'

    def __init__(self, context, request):
        super(JsonpPollingTransport, self).__init__(
            context,
            request,
            False
        )
        self.session = None
        self.active = True
        self.callback = None

    def get(self):
        response = self.response

        # Start response
        self.handle_session_cookie()
        self.disable_cache()

        # Grab callback parameter
        self.callback = self.request.GET.get('c', None)
        if self.callback is None:
            return HTTPServerError('"callback" parameter required')

        session_id = self.sid = self.context.parent.re_matched_name

        # Get or create session without starting heartbeat
        session = self.context.sockjs_server.get_session(self.sid)
        if session is None:
            session = self.context.sockjs_server.create_session(session_id)
        if session is None:
            return HTTPServerError("ERROR creating session.")

        # Try to attach to the session
        if not session.set_handler(self, False):
            return response
        self.session = session
        # Verify if session is properly opened
        session.verify_state()

        # Might get already detached because connection was closed in on_open
        if not self.session:
            return response

        if not self.session.send_queue:
            self.session.start_heartbeat()
        else:
            self.session.flush()
        return response

    def send_pack(self, message, binary=False):
        if binary:
            raise Exception('binary not supported for JsonpPollingTransport')

        if not self.active:
            return
        self.active = False

        try:
            self.response.content_type = 'application/javascript'
            self.response.charset = 'UTF-8'
            # TODO: Fix me
            self.set_header('Etag', 'dummy')

            # TODO: Just escape
            msg = '%s(%s);\r\n' % (self.callback, proto.json_encode(message))
            self.write(msg)
            self.flush()
            self.send_complete()
        except Exception:
            # If connection dropped, make sure we close offending session
            # instead of propagating error all way up.
            logging.exception(
                "ERROR JsonpPollingTransport send_pack")
            if self.session:
                self.session.delayed_close()
            raise

    def send_complete(self):
        if self.session:  # detach session
            self.session.remove_handler(self)
            self.session = None
        self.active = True

    def session_closed(self):
        """Called by the session when it was closed"""
        if self.session:  # detach session
            self.session.remove_handler(self)
            self.session = None
        self.context.gaplic.add_callback(self.safe_finish)



import unittest

from ginsfsm.gobj import (
    GObj,
    EventNotAcceptedError,
    EventError,
#    StateError,
    GObjError,
    )

import logging
logging.basicConfig(level=logging.DEBUG)

########################################################
#       Child
########################################################


def ac_query_and_direct_response(self, event):
    data = event.data
    if data == 'query1':
        return self.send_event(self.parent, 'EV_RESP_OK', response='OK')
    else:
        return self.send_event(self.parent, 'EV_RESP_ERROR', response='ERROR')


def ac_query_and_broadcast_response(self, event):
    data = event.data
    if data == 'query3':
        self.broadcast_event('EV_RESP_OK', response='OK')
    else:
        self.broadcast_event('EV_RESP_ERROR', response='ERROR')
    event.event_name = 'EV_RESP_OK'
    self.send_event(self.parent, event)


FSM_CHILD = {
    'event_list': (
        'EV_QUERY_BY_DIRECT',
        'EV_QUERY_BY_BROADCAST',
        'EV_RESP_OK:output',
        'EV_RESP_ERROR:output',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_QUERY_BY_DIRECT',    ac_query_and_direct_response,    None),
            ('EV_QUERY_BY_BROADCAST', ac_query_and_broadcast_response, None),
        ),
    }
}


class ChildGClass(GObj):
    def __init__(self):
        GObj.__init__(self, FSM_CHILD)
        self.start_up()

    def start_up(self):
        ''' Create gobj childs, initialize something...
        '''

########################################################
#       Parent
########################################################


def ac_response_ok(self, event):
    self.response = event.response
    return 'Done'


def ac_response_rename_ok(self, event):
    self.response = event.response + 'renamed'
    self.original_event_name = event.original_event_name
    return 'Done'


def ac_response_error(self, event):
    self.response = event.response
    return 'Done'


def ac_consult(self, event):
    return self.send_event(self.cons, event)


FSM_PARENT = {
    'event_list': (
        'EV_QUERY_BY_DIRECT',
        'EV_QUERY_BY_BROADCAST',
        'EV_RESP_OK',
        'EV_RESP_RENAME_OK',
        'EV_RESP_ERROR',
    ),
    'state_list': (
        'ST_IDLE',
        'ST_WAIT_RESP',
    ),
    'machine': {
        'ST_IDLE':
        (
            ('EV_QUERY_BY_DIRECT',      ac_consult,     'ST_WAIT_RESP'),
            ('EV_QUERY_BY_BROADCAST',   ac_consult,     'ST_WAIT_RESP'),
        ),

        'ST_WAIT_RESP':
        (
            ('EV_RESP_OK',          ac_response_ok,           'ST_IDLE'),
            ('EV_RESP_RENAME_OK',   ac_response_rename_ok,    'ST_IDLE'),
            ('EV_RESP_ERROR',       ac_response_error,        'ST_IDLE'),
        ),
    }
}


class ParentGClass(GObj):
    def __init__(self):
        GObj.__init__(self, FSM_PARENT)
        self.start_up()

    def start_up(self):
        ''' Create gobj childs, initialize something...
        '''
        self.cons = ChildGClass()
        self._add_child(self.cons)


########################################################
#       Tests
########################################################
class TestGObj(unittest.TestCase):
    def setUp(self):
        self.gobj_parent = ParentGClass()
        settings = {
            'GObj.trace_mach': True,
            'GObj.logger': logging,
        }
        self.gobj_parent.overwrite_parameters(-1, **settings)

    def test_create_gobj(self):
        self.assertRaises(GObjError,
            self.gobj_parent.create_gobj, None, None, None)
        self.assertRaises(TypeError,
            self.gobj_parent.create_gobj, None, {}, None)
        self.assertRaises(GObjError,
            self.gobj_parent.create_gobj, None, {}.__class__, None)
        self.assertRaises(GObjError,
            self.gobj_parent.create_gobj, None, ChildGClass, {})

    def test_destroy_gobj(self):
        childs1 = len(self.gobj_parent.dl_childs)
        child = self.gobj_parent.create_gobj(
            None, ChildGClass, self.gobj_parent)
        childs2 = len(self.gobj_parent.dl_childs)
        self.assertEqual(childs1, childs2 - 1)
        self.gobj_parent.destroy_gobj(child)
        childs3 = len(self.gobj_parent.dl_childs)
        self.assertEqual(childs1, childs3)

    def test_send_event_name_to_itself(self):
        ret = self.gobj_parent.send_event(self.gobj_parent,
            'EV_QUERY_BY_DIRECT', data='query1')
        self.assertEqual(ret, 'Done')
        self.assertEqual(self.gobj_parent.response, 'OK')

        ret = self.gobj_parent.send_event(self.gobj_parent,
            'EV_QUERY_BY_DIRECT', data='query2')
        self.assertEqual(ret, 'Done')
        self.assertEqual(self.gobj_parent.response, 'ERROR')

    def test_send_event_to_itself(self):
        ret = self.gobj_parent.send_event(self.gobj_parent,
            'EV_QUERY_BY_DIRECT', data='query1')
        self.assertEqual(ret, 'Done')
        self.assertEqual(self.gobj_parent.response, 'OK')

        ret = self.gobj_parent.send_event(self.gobj_parent,
            'EV_QUERY_BY_DIRECT', data='query2')
        self.assertEqual(ret, 'Done')
        self.assertEqual(self.gobj_parent.response, 'ERROR')

    def test_send_event_name_to_child(self):
        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query1')
        self.assertEqual(ret, EventNotAcceptedError)

        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query2')
        self.assertEqual(ret, EventNotAcceptedError)

    def test_send_event_to_child(self):
        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query1')
        self.assertEqual(ret, EventNotAcceptedError)

        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query2')
        self.assertEqual(ret, EventNotAcceptedError)

    def test_send_event_name_to_none(self):
        self.assertRaises(GObjError, self.gobj_parent.send_event,
            None, 'EV_QUERY_BY_DIRECT', data='query1')
        self.assertRaises(GObjError, self.gobj_parent.send_event,
            'pepe', 'EV_QUERY_BY_DIRECT', data='query1')

    def test_send_event_to_none(self):
        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query1')
        self.assertEqual(ret, EventNotAcceptedError)

        ret = self.gobj_parent.send_event(self.gobj_parent.cons,
            'EV_QUERY_BY_DIRECT', data='query2')
        self.assertEqual(ret, EventNotAcceptedError)

    def test_send_event_to_named_gobj(self):
        self.assertRaises(GObjError, self.gobj_parent.send_event,
            'destination', 'EV_QUERY_BY_DIRECT')

    def test_event_factory(self):
        self.assertRaises(EventError, self.gobj_parent._event_factory,
            self.gobj_parent, 1)
        self.assertRaises(GObjError, self.gobj_parent._event_factory,
            1, 'EV_QUERY_BY_DIRECT')

        event = self.gobj_parent._event_factory(
            'pepe',
            'EV_QUERY_BY_DIRECT',
            data='query1')
        self.assertRaises(GObjError, self.gobj_parent._event_factory,
            1, event)

        event = self.gobj_parent._event_factory('pepe', event, data2='query2')
        self.assertEqual(event.destination, 'pepe')
        self.assertEqual(event.data, 'query1')
        self.assertEqual(event.data2, 'query2')
        event = self.gobj_parent._event_factory(self.gobj_parent, event)
        self.assertEqual(event.destination, self.gobj_parent)

    def test_post_event(self):
        self.assertRaises(GObjError, self.gobj_parent.post_event,
            self.gobj_parent, 'EV_QUERY_BY_DIRECT', data='query1')

        self.assertRaises(GObjError, self.gobj_parent.post_event,
            self.gobj_parent, 'EV_QUERY_BY_DIRECT', data='query1')

    def test_broadcast_event1(self):
        self.gobj_parent.cons.subscribe_event(
            'EV_RESP_OK', self.gobj_parent)
        ln = len(self.gobj_parent.cons._dl_subscriptions)
        self.assertEqual(ln, 1)

        self.gobj_parent.cons.subscribe_event(
            'EV_RESP_OK', self.gobj_parent)
        ln = len(self.gobj_parent.cons._dl_subscriptions)
        self.assertEqual(ln, 1)

        self.gobj_parent.send_event(
            self.gobj_parent, 'EV_QUERY_BY_BROADCAST', data='query3')
        self.assertEqual(self.gobj_parent.response, 'OK')
        self.gobj_parent.cons.delete_subscription(
            'EV_RESP_OK', self.gobj_parent)
        ln = len(self.gobj_parent.cons._dl_subscriptions)
        self.assertEqual(ln, 0)

    def test_broadcast_event2(self):
        self.gobj_parent.cons.subscribe_event(
            ('EV_RESP_OK', 'EV_RESP_ERROR'), self.gobj_parent)
        self.gobj_parent.send_event(self.gobj_parent,
                'EV_QUERY_BY_BROADCAST', data='query4')
        self.assertEqual(self.gobj_parent.response, 'ERROR')
        self.gobj_parent.cons.delete_subscription(
            ('EV_RESP_OK', 'EV_RESP_ERROR'), self.gobj_parent)

    def test_broadcast_event3(self):
        self.gobj_parent.cons.subscribe_event(
            None, self.gobj_parent)
        self.gobj_parent.send_event(self.gobj_parent,
                'EV_QUERY_BY_BROADCAST', data='query4')
        self.assertEqual(self.gobj_parent.response, 'ERROR')
        self.gobj_parent.cons.delete_subscription(
            None, self.gobj_parent)

    def test_broadcast_event4(self):
        self.gobj_parent.cons.subscribe_event(
            (None,), self.gobj_parent)
        self.gobj_parent.send_event(self.gobj_parent,
                'EV_QUERY_BY_BROADCAST', data='query4')
        self.assertEqual(self.gobj_parent.response, 'ERROR')
        self.gobj_parent.cons.delete_subscription(
            (None,), self.gobj_parent)

    def test_broadcast_event5(self):
        self.gobj_parent.cons.subscribe_event(
            'EV_RESP_OK',
            self.gobj_parent,
            __rename_event_name__='EV_RESP_RENAME_OK',
        )
        self.gobj_parent.send_event(
            self.gobj_parent, 'EV_QUERY_BY_BROADCAST', data='query3')
        self.assertEqual(self.gobj_parent.response, 'OKrenamed')
        self.assertEqual(self.gobj_parent.original_event_name, 'EV_RESP_OK')

    def test_subscribe_event_and_delete_subscription(self):
        self.assertRaises(GObjError, self.gobj_parent.cons.subscribe_event,
            'EV_RESP_OK', 1)
        self.assertRaises(EventError, self.gobj_parent.cons.subscribe_event,
            'EV_XXX', self.gobj_parent)

        self.gobj_parent.cons.subscribe_event(
            'EV_RESP_OK', self.gobj_parent
        )
        self.assertTrue(self.gobj_parent.cons.delete_subscription(
            'EV_RESP_OK', self.gobj_parent))

        self.assertFalse(self.gobj_parent.cons.delete_subscription(
            'EV_RESP_OK', self.gobj_parent))

        self.gobj_parent.cons.subscribe_event(
            ('EV_RESP_OK', 'EV_RESP_ERROR'), self.gobj_parent
        )
        self.assertTrue(self.gobj_parent.cons.delete_subscription(
            ('EV_RESP_OK', 'EV_RESP_ERROR'), self.gobj_parent))

        self.assertRaises(EventError, self.gobj_parent.cons.subscribe_event,
            ('EV_RESP_OK', 1), self.gobj_parent)

    def test_add_child(self):
        self.assertRaises(GObjError,
            self.gobj_parent._add_child, self.gobj_parent.cons)

    def test_remove_child(self):
        self.gobj_parent._remove_child(self.gobj_parent.cons)



import unittest
from ginsfsm.deferred import (
    Deferred,
    DeferredList,
    )


def func1(arg1, kwarg1=None):
    return (arg1, kwarg1)


def func2(*args, **kwargs):
    return (args, kwargs)


def func3(dst, event):
    def _func3(dst=dst, event=event):
        return (dst, event)
    return _func3


class TestDeferred(unittest.TestCase):
    def setUp(self):
        self.deferred_list = DeferredList()

    def test_callback1(self):
        self.deferred_list.add_callback(func1, func1, 1, kwarg1=2)
        result = self.deferred_list(func1)
        x, y = result
        self.assertEqual(x, 1)
        self.assertEqual(y, 2)

    def test_callback2(self):
        self.deferred_list.add_callback("refXXX", func2, 2, kwarg1=1, kwarg2=2)
        result = self.deferred_list("refXXX", kwarg3=3)
        l, d = result
        self.assertEqual(l[0], 2)
        self.assertEqual(d['kwarg1'], 1)
        self.assertEqual(d['kwarg2'], 2)
        self.assertEqual(d['kwarg3'], 3)

    def test_callback3(self):
        self.deferred_list.add_callback(None, func2, 2, kwarg1=1, kwarg2=2)
        result = self.deferred_list(None, kwarg3=3)
        l, d = result
        self.assertEqual(l[0], 2)
        self.assertEqual(d['kwarg1'], 1)
        self.assertEqual(d['kwarg2'], 2)
        self.assertEqual(d['kwarg3'], 3)

    def test_callback4(self):
        self.deferred_list.add_callback('XXX',
                                        func3(dst='gobj1', event='ev_event1'))
        result = self.deferred_list('XXX')
        d, e = result
        self.assertEqual(d, 'gobj1')
        self.assertEqual(e, 'ev_event1')

        result = self.deferred_list('XXX', dst='gobj2')
        d, e = result
        self.assertEqual(d, 'gobj2')
        self.assertEqual(e, 'ev_event1')



# -*- encoding: utf-8 -*-
""" basehandler
"""

from datetime import datetime, timedelta

from pyramid.response import Response

from ginsfsm.protocols.sockjs.server.session import ConnectionInfo
from ginsfsm.protocols.wsgi.webob.async_response import AsyncResponse
from ginsfsm.deferred import Deferred
from ginsfsm.compat import bytes_

#----------------------------------------------------------------#
#                   Base Views
#----------------------------------------------------------------#
CACHE_TIME = 31536000


class BaseHandler(object):
    def __init__(self, context, request, async_response=False):
        self.context = context
        self.request = request
        self.gsock = request.environ['ginsfsm.gsock']
        self.async_response = async_response
        if async_response:
            self.response = AsyncResponse()
        else:
            self.response = Response()

    # Various helpers
    def set_header(self, name, value):
        """Sets the given response header name and value.
        """
        self.response.headerlist.append((name, value))

    def set_status(self, code):
        self.response.status = code

    def set_body(self, body):
        if self.async_response:
            pass  # be care writing to body in async mode
        self.response.body = bytes_(body, "utf-8")

    def enable_cache(self):
        """Enable client-side caching for the current request"""
        d = datetime.now() + timedelta(seconds=CACHE_TIME)
        self.set_header('Cache-Control', 'max-age=%d, public' % CACHE_TIME)
        self.set_header('Expires', d.strftime('%a, %d %b %Y %H:%M:%S'))
        self.set_header('Access-Control-Max-Age', str(CACHE_TIME))

    def disable_cache(self):
        """Disable client-side cache for the current request"""
        self.set_header(
            'Cache-Control',
            'no-store, no-cache, must-revalidate, max-age=0'
        )

    def handle_session_cookie(self):
        """Handle JSESSIONID cookie logic"""
        # Tornado:
        # If JSESSIONID support is disabled in the settings,
        # ignore cookie logic
        if not self.context.sockjs_server.config.jsessionid:
            return

        cookie = self.request.cookies.get('JSESSIONID')

        if not cookie:
            cookie = 'dummy'

        self.response.set_cookie('JSESSIONID', cookie)
        return ('Set-Cookie', self.response.headers['Set-Cookie'])

    def write(self, data):
        """ Write data.
            It will use the configurated response: async or sync.
        """
        self.response.write(bytes_(data))

    def flush(self, callback=None):
        """Flushes the current output buffer to the network.

        The ``callback`` argument, if given, can be used for flow control:
        it will be run when all flushed data has been written to the socket.
        Note that only one flush callback can be outstanding at a time;
        if another flush occurs before the previous flush's callback
        has been run, the previous callback will be discarded.
        """
        if self.async_response:
            self.response.flush(callback)
        else:
            if callback:
                self.context.gaplic.add_callback(callback)

    def finish(self):
        """Finishes this response, ending the HTTP request."""
        if self.async_response:
            self.response.finish()
        else:
            if self.gsock:
                gsock = self.gsock
                self.gsock = None
                gsock.mt_drop()

    def safe_finish(self):
        """ Finish session. Ignore any error.
        """
        try:
            self.finish()
        except:
            pass

    def get_conn_info(self):
        """Return `ConnectionInfo` object from current transport"""
        return ConnectionInfo(
            self.request.remote_addr,  # remote_ip
            self.request.cookies,
            self.request.params,  # arguments
            self.request.headers,
            self.request.path
        )


class PreflightHandler(BaseHandler):
    """CORS preflight handler"""

    def __init__(self, context, request, async_response=False):
        super(PreflightHandler, self).__init__(
            context, request, async_response
        )

    def options(self, *args, **kwargs):
        """XHR cross-domain OPTIONS handler"""
        # Force a clean sync Response.
        # Several views point here: both async and sync responses.
        response = self.response = Response()

        self.enable_cache()
        self.handle_session_cookie()
        self.preflight()

        if self.verify_origin():
            allowed_methods = getattr(self, 'access_methods', 'OPTIONS, POST')
            self.set_header('Access-Control-Allow-Methods', allowed_methods)
            self.set_header('Allow', allowed_methods)
            self.set_status(204)
        else:
            # Set forbidden
            self.set_status(403)

        return response

    def preflight(self):
        """ Handles request authentication
            Cors headers.
        """
        request = self.request
        # origin = self.request.headers.get('Origin', '*') tornado
        origin = request.environ.get("HTTP_ORIGIN", '*')
        if origin == 'null':
            origin = '*'  # Respond with '*' to 'null' origin

        self.set_header('Access-Control-Allow-Origin', origin)

        # headers = self.request.headers.get('Access-Control-Request-Headers')
        headers = request.environ.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS')
        if headers:
            self.set_header('Access-Control-Allow-Headers', headers)

        self.set_header('Access-Control-Allow-Credentials', 'true')

    def verify_origin(self):
        """Verify if request can be served"""
        # TODO: Verify origin
        return True


class SessionHandler(PreflightHandler):
    """Session handler"""

    # TODO: pasa todo lo de sessio aquí!

    def __init__(self, context, request, async_response=False):
        super(SessionHandler, self).__init__(
            context, request, async_response
        )
        self.session = None
        deferred = Deferred(0, self.on_connection_close)
        self.gsock.subscribe_event('EV_DISCONNECTED', deferred)

    def on_connection_close(self, event=None):
        # If connection was dropped by the client, close session.
        # In all other cases, connection will be closed by the server.
        if self.session is not None:
            self.session.close(1002, 'Connection interrupted')



# -*- encoding: utf-8 -*-
""" Cuentas GObj

.. autoclass:: Cuentas
    :members: start_up

"""

from ginsfsm.gobj import GObj


def ac_cuenta(self, event):
    """ Event attributes:
        * :attr:`user_name`: user name.

    """
    user_name = event.user_name
    if not user_name in self.cuentas:
        self.cuentas[user_name] = 0

    if event.__intra_event__:
        origin_gaplic = event.__intra_event__.origin_gaplic
        origin_role = event.__intra_event__.origin_role
        origin_gobj = event.__intra_event__.origin_gobj

        self.gaplic.send_event_outside(
            origin_gaplic,  # gaplic_name
            origin_role,  # role
            origin_gobj,  # gobj_name
            'EV_CUENTA_ACK',  # event_name
            self,  # subscriber_gobj
        )


CUENTAS_FSM = {
    'event_list': (
        'EV_CUENTA:top input',
        'EV_CUENTA_ACK:top output',
    ),
    'state_list': (
        'ST_IDLE',
    ),
    'machine': {
        'ST_IDLE':
        (
            ('EV_CUENTA',      ac_cuenta,      None),
        ),
    }
}

CUENTAS_GCONFIG = {
    'subscriber': [
        None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
    ],
}


class Cuentas(GObj):
    """  Cuentas GObj.
    A Sample gobj.

    .. ginsfsm::
       :fsm: CUENTAS_FSM
       :gconfig: CUENTAS_GCONFIG

    *Input-Events:*
        * :attr:`'EV_CUENTA'`: sample input event.

          Event attributes:

              * ``data``: sample event attribute.

    *Output-Events:*
        * :attr:`'EV_CUENTA_ACK'`: sample output event.

          Event attributes:

              * ``data``: sample event attribute.

    """
    def __init__(self):
        GObj.__init__(self, CUENTAS_FSM, CUENTAS_GCONFIG)
        self.cuentas = {}

    def start_up(self):
        """ Initialization zone.
        """
        if self.config.subscriber:
            self.subscribe_event(None, self.config.subscriber)



# -*- coding: utf-8 -*-
from ginsfsm.compat import (
    urlparse,
    text_type,
    bytes_,
)
from time import time
from hashlib import md5
from random import random


def random_key():
    """Return random session key"""
    i = md5()
    i.update(bytes_('%s%s' % (random(), time())))
    return i.hexdigest()


def string_to_bytearray(s, encoding='latin-1', errors='strict'):
    if isinstance(s, text_type):
        s = bytearray(s, encoding, errors)
    elif not isinstance(s, bytearray):
        s = bytearray(s)
    return s


def hexdump(prefix, byts, length=16):
    ''' hd: hexdump
    dump in hex with pretty formating the hex value and ascii value (if any)
    for a block of bytes [assumed to be a tuple]

    byts: incoming bytes
    length: how many bytes to display on each line.
    '''
    byts = string_to_bytearray(byts)
    n = 0
    result = ''
    while byts:
        b_work = byts[:length]
        byts = byts[length:]
        hexa = ' '.join(["%02X" % x for x in b_work])
        asc = ''.join([((((x < 32) or (x > 0x7e)) and '.') or chr(x)) for x in b_work])
        result += "%s %04X %-*s %s\n" % (prefix, n, length*3, hexa, asc)
        n += length
    return result


def get_path_from_url(url):
    o = urlparse.urlparse(url)
    return o.path


def get_host_port_from_urls(urls):
    routes = []
    for url in urls:
        o = urlparse.urlparse(url)
        if not o.hostname or not o.scheme:
            print("Can't get HOST from URL %r" % url)
            continue
        if not o.port:
            if o.scheme in ('wss', 'https'):
                port = 443
            elif o.scheme in ('ws', 'http'):
                port = 80
            else:
                port = 80
        else:
            port = o.port
        if not port:
            print("Can't get PORT from URL %r" % url)
            continue

        routes.append((o.hostname, port),)
    return routes



# -*- encoding: utf-8 -*-
"""
    AsyncResponse -  an asynchronous WebOb Response over ginsfsm.
"""
import logging

from ginsfsm.protocols.http.server.c_http_clisrv import ResponseInterrupt

try:
    from pyramid.response import Response
except ImportError:  # pragma: no cover
    try:
        from webob import Response
    except ImportError:  # pragma: no cover
        raise Exception('async_response is depending of Pyramid or WebOb')


class AsyncResponse(Response):
    def __init__(self, **kw):
        super(AsyncResponse, self).__init__(**kw)
        self.ginsfsm_channel = None

        del self.app_iter  # Important trick to remove Content-Length.
        # WARNING: you cannot use body, because it sets Content-Length.

    def __call__(self, environ, start_response):
        "Override WSGI call"
        self.ginsfsm_channel = environ['ginsfsm.channel']

        start_response(self.status, self._abs_headerlist(environ))

        # send the current body to the network
        for chunk in self.app_iter:
            self.write(chunk)
            self.flush()
        del self.app_iter

        raise ResponseInterrupt()

    def write(self, data):
        """ Write data to http channel.
        """
        if self.ginsfsm_channel:
            self.ginsfsm_channel.write(data)
        else:
            # write in the sync response by now.
            # in wsgi response call, all app_iter will be flush.
            super(AsyncResponse, self).write(data)

    def flush(self, callback=None):
        """Flushes the current output buffer to the network.

        The ``callback`` argument, if given, can be used for flow control:
        it will be run when all flushed data has been written to the socket.
        Note that only one flush callback can be outstanding at a time;
        if another flush occurs before the previous flush's callback
        has been run, the previous callback will be discarded.
        """
        if self.ginsfsm_channel:
            self.ginsfsm_channel.flush(callback)
        else:
            logging.error('ERROR async Flush before set ginsfsm_channel')

    def finish(self):
        """Finishes this response, ending the HTTP request."""
        if self.ginsfsm_channel:
            self.ginsfsm_channel.finish()
        else:
            logging.error('ERROR async FINISH before set ginsfsm_channel')



# -*- encoding: utf-8 -*-
"""
Routing events between gaplics.
===============================

You can run this file with ``gserve test_router.ini``

.. autoclass:: Sample1
    :members:

.. autoclass:: Sample2
    :members:


In this example:
    TITI gaplic of main thread
    TOTO gaplic another thread or subprocess

TOTO enable router server in port 8000
TITI add static router to localhost:8000

TOTO has EV_SET_TIMEOUT Api.
    Its action is set a timeout that broadcast EV_MESSAGE.

TITI do two things:
    - subscribe to event EV_MESSAGE of external TOTO
    - send EV_SET_TIMEOUT to external TOTO

    The EV_SET_TIMEOUT event reachs TOTO,
    then it set the timer that broadcast EV_MESSAGE.

    TITI receive the external EV_MESSAGE because it's subscribe it.

    When TITI receive the EV_MESSAGE,
    re-send EV_SET_TIMEOUT to external TOTO
    and so on.

"""
import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer
from ginsfsm.gobj import GObj
from ginsfsm.globals import (
    list_globals,
)


#===============================================================
#                   GAplic TITI
#===============================================================
def ac_message(self, event):
    print("ORDENNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN %s %s" % (
        self.counter,
        event.__subscription_reference__,
    ))
    list_globals()
    self.counter += 1
    if self.counter > 4:
        # delete subscription
        ret = self.gaplic.unsubscribe_event_outside(
            'TOTO',         # gaplic_name
            '',             # role
            'sample2',      # gobj_name
            'EV_MESSAGE',   # event_name
            self,           # subscriber_gobj
            __subscription_reference__=event.__subscription_reference__,
        )
        if not ret:
            print("Eiiiiiiiiiiiiiiiiiiiiii something wrong!")

    # causes broadcast EV_MESSAGE again
    self.gaplic.send_event_outside(
        'TOTO',             # gaplic_name
        '',                 # role
        'sample2',          # gobj_name
        'EV_SET_TIMEOUT',   # event_name
        self,               # subscriber_gobj
        timeout=2,
    )


SAMPLE1_FSM = {
    'event_list': (
        'EV_MESSAGE: top input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_MESSAGE', ac_message, None),
        ),
    }
}

SAMPLE1_GCONFIG = {  # type, default_value, flag, validate_function, desc
}


class Sample1(GObj):
    """  Sample1 GObj.

    .. ginsfsm::
       :fsm: SAMPLE1_FSM
       :gconfig: SAMPLE1_GCONFIG

    *Input-Events:*

        * :attr:`'EV_MESSAGE'`: .

    *Output-Events:*

    """
    def __init__(self):
        GObj.__init__(self, SAMPLE1_FSM, SAMPLE1_GCONFIG)
        self.counter = 0

    def start_up(self):
        """
        """
        ret = self.gaplic.subscribe_event_outside(
            'TOTO',         # gaplic_name
            '',             # role
            'sample2',      # gobj_name
            'EV_MESSAGE',   # event_name
            self,           # subscriber_gobj
        )
        if not ret:
            print("Eiiiiiiiiiiiiiiiiiiiiii something wrong!")

        self.gaplic.send_event_outside(
            'TOTO',             # gaplic_name
            '',                 # role
            'sample2',          # gobj_name
            'EV_SET_TIMEOUT',   # event_name
            self,               # subscriber_gobj
            timeout=2,
        )


#===============================================================
#                   GAplic TOTO
#===============================================================
def ac_set_timeout(self, event):
    # TODO: get the seconds
    timeout = event.timeout
    self.set_timeout(timeout)


def ac_timeout(self, event):
    print("Broadcasting EV_MESSAGE")
    self.broadcast_event('EV_MESSAGE')


SAMPLE2_FSM = {
    'event_list': (
        'EV_MESSAGE: top output',
        'EV_TIMEOUT: bottom input',
        'EV_SET_TIMEOUT: top input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_SET_TIMEOUT',  ac_set_timeout,     None),
            ('EV_TIMEOUT',      ac_timeout,         None),
        ),
    }
}

SAMPLE2_GCONFIG = {  # type, default_value, flag, validate_function, desc
}


class Sample2(GObj):
    """  Sample2 GObj.

    .. ginsfsm::
       :fsm: SAMPLE2_FSM
       :gconfig: SAMPLE2_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: .

    *Output-Events:*

        * :attr:`'EV_MESSAGE'`: .

    """
    def __init__(self):
        GObj.__init__(self, SAMPLE2_FSM, SAMPLE2_GCONFIG)
        self.count = 0

    def start_up(self):
        """
        """
        self.timer = self.create_gobj(
            None,
            GTimer,
            self)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Main
#===============================================================
def gaplic1(global_config, **local_conf):
    """ Entry point to run with gserve (PasteDeploy)
    """
    if 'gaplic-name' in local_conf:
        gaplic_name = local_conf.pop('gaplic-name')
    else:
        raise Exception('You must supply an gaplic name ("gaplic-name")')

    ga = GAplic(name=gaplic_name, roles=('titi',), **local_conf)
    ga.create_gobj(
        'sample1',
        Sample1,
        ga,
        __unique_name__=True,
    )
    return ga


def gaplic2(global_config, **local_conf):
    """ Entry point to run with gserve (PasteDeploy)
    """
    if 'gaplic-name' in local_conf:
        gaplic_name = local_conf.pop('gaplic-name')
    else:
        raise Exception('You must supply an gaplic name ("gaplic-name")')

    ga = GAplic(name=gaplic_name, roles=('toto',), **local_conf)
    ga.create_gobj(
        'sample2',
        Sample2,
        ga,
        __unique_name__=True,
    )
    return ga


if __name__ == "__main__":
    """ You can run directly this file, without gserve.
    """
    import logging
    logging.basicConfig(level=logging.DEBUG)
    from ginsfsm.gaplic import setup_gaplic_thread, setup_gaplic_process

    run_as_process = False

    # simulate running from ini file
    local_conf = {
        'gaplic-name': 'TOTO',
        'router_enabled': True,
        'GRouter.server': True,
        'GRouter.trace_router': False,
        'GRouter.localhost_route_ports': 8000,
        'GSock.trace_dump': False,
        'GObj.trace_mach': False,
        'GObj.logger': logging,
    }
    if run_as_process:
        worker = setup_gaplic_process(gaplic2({}, **local_conf))
        worker.start()
    else:
        worker = setup_gaplic_thread(gaplic2({}, **local_conf))
        worker.start()

    # run main process
    local_conf = {
        'gaplic-name': 'TITI',
        'router_enabled': True,
        'GRouter.server': False,
        'GRouter.trace_router': False,
        'GRouter.static_routes': 'TOTO, toto, http://localhost:8000',
        'GSock.trace_dump': False,
        'GObj.trace_mach': False,
        'GObj.logger': logging,
    }
    ga = gaplic1({}, **local_conf)
    try:
        ga.start()
    except (KeyboardInterrupt, SystemExit):

        # stop the gaplic
        worker.stop()

         # wait to finish the other gaplic
        worker.join()

        print('Program stopped')



# -*- encoding: utf-8 -*-
"""
GObj :class:`GSock`
===================

GObj for manage socket events.

.. autoclass:: GSock
    :members: start_up, get_next_dst, get_peername, get_sockname

"""

# ======================================================================
#   Code inspired in asyncore.py, waitress and tornado.
# ======================================================================

try:
    import ssl  # Python 2.6+
except ImportError:
    ssl = None

import select
import traceback
import datetime
import socket
import time
import errno
import sys
import warnings
import os

from ginsfsm.gobj import GObj
from ginsfsm.gconfig import GConfig
from ginsfsm.compat import text_type
from ginsfsm.utils import (
    hexdump,
)
from ginsfsm.buffers import (
    ReadOnlyFileBasedBuffer,
    OverflowableBuffer,
)
from errno import (
    EALREADY,
    EINPROGRESS,
    EWOULDBLOCK,
    ECONNRESET,
    EINVAL,
    ENOTCONN,
    ESHUTDOWN,
    #    EINTR,
    EISCONN,
    EBADF,
    ECONNABORTED,
    EAGAIN,
    #    EADDRNOTAVAIL,
    errorcode
)

_DISCONNECTED = frozenset((
    ECONNRESET,
    ENOTCONN,
    ESHUTDOWN,
    ECONNABORTED,
))


class IOLoop(object):
    # Constants from the epoll module
    _EPOLLIN = 0x001
    _EPOLLPRI = 0x002
    _EPOLLOUT = 0x004
    _EPOLLERR = 0x008
    _EPOLLHUP = 0x010
    _EPOLLRDHUP = 0x2000
    _EPOLLONESHOT = (1 << 30)
    _EPOLLET = (1 << 31)

    # Our events map exactly to the epoll events
    NONE = 0
    READ = _EPOLLIN
    WRITE = _EPOLLOUT
    ERROR = _EPOLLERR | _EPOLLHUP | _EPOLLRDHUP


def _strerror(err):
    try:
        return os.strerror(err)
    except (ValueError, OverflowError, NameError):
        if err in errorcode:
            return errorcode[err]
        return "Unknown error %s" % err

_reraised_exceptions = (
    KeyboardInterrupt,
    SystemExit,
)


def readwrite(obj, flags):
    try:
        if flags & (IOLoop.ERROR):
            obj.handle_close()
            return
        if flags & (IOLoop.READ):
            obj.handle_read_event()
        if flags & (IOLoop.WRITE):
            obj.handle_write_event()
    except socket.error as e:
        if e.args[0] not in (
                EBADF,
                ECONNRESET,
                ENOTCONN,
                ESHUTDOWN,
                ECONNABORTED):
            obj.handle_error()
        else:
            obj.handle_close()
    except _reraised_exceptions:
        close_all_sockets(obj._socket_map)
        raise
    except:
        obj.logger.error(
            "Exception in I/O handler for fd %r", obj, exc_info=True)


def poll_loop(socket_map, _impl, timeout):
    """ check poll, return True if some sock event, otherwise False.
    """
    ret = False
    try:
        event_pairs = _impl.poll(timeout)
    except Exception as e:
        # Depending on python version and IOLoop implementation,
        # different exception types may be thrown and there are
        # two ways EINTR might be signaled:
        # * e.errno == errno.EINTR
        # * e.args is like (errno.EINTR, 'Interrupted system call')
        if (getattr(e, 'errno', None) == errno.EINTR or
                (isinstance(getattr(e, 'args', None), tuple) and
                    len(e.args) == 2 and e.args[0] == errno.EINTR)):
            return ret
        else:
            raise
    #print "events ------------->", event_pairs
    #print "sockmp ------------->", socket_map.keys()
    for fd, events in event_pairs:
        obj = socket_map.get(fd, None)
        if obj is None:
            continue
        ret = True
        readwrite(obj, events)
    return ret


#===========================================================
#       GSock gobj
#===========================================================
def ac_connect(self, event):
    self._mt_connect(**event.kw)


def ac_drop(self, event):
    self.mt_drop()


def ac_send_data(self, event):
    """ Write in the output data buffer and flush it right now.
    """
    # TODO: callback when transmit_ready
    self.mt_send_data(event.data)


def ac_write(self, event):
    """ Write in the output data buffer.
        The data are not really sent until _send_some() is called.
    """
    self.mt_write(event.data)


def ac_flush(self, event):
    """ Flush the output data buffer.
        Equivalent to waitress' send_some().
    """
    # TODO: callback when transmit_ready
    self.mt_flush()


def ac_disconnected(self, event):
    """ Used by GServerSock.
    """
    gsock = event.gsock
    self.post_event(self, 'EV_DESTROY_GSOCK', gsock=gsock)


def ac_drop_gsock(self, event):
    """ Used by GServerSock.
        Destroy gsock.
    """
    gsock = event.gsock
    GObj.destroy_gobj(gsock)
    self.config.n_gsocks -= 1


GSOCK_FSM = {
    'event_list': (
        'EV_CONNECT:top input',
        'EV_DROP:top input',
        'EV_CONNECTED: top output',
        'EV_DISCONNECTED: top output: bottom input',  # used by srvsock
        'EV_RX_DATA: top output',
        'EV_SEND_DATA: top input',
        'EV_WRITE_OUTPUT_DATA: top input',
        'EV_FLUSH_OUTPUT_DATA: top input',
        'EV_TRANSMIT_READY: top output',
        'EV_DESTROY_GSOCK',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_CONNECT',              ac_connect,         None),
            ('EV_DROP',                 ac_drop,            None),
            ('EV_SEND_DATA',            ac_send_data,       None),
            ('EV_WRITE_OUTPUT_DATA',    ac_write,           None),
            ('EV_FLUSH_OUTPUT_DATA',    ac_flush,           None),

            # Used by GServerSock
            ('EV_DISCONNECTED',         ac_disconnected,    None),
            ('EV_DESTROY_GSOCK',        ac_drop_gsock,      None),
        ),
    }
}

GSOCK_GCONFIG = {
    'subscriber': [
        None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
    ],
    'trace_dump': [
        bool, False, GConfig.FLAG_DIRECT_ATTR, None,
        "trace tx/rx messages"
    ],
    'ip': [str, '', 0, None, ""],
    'host': [str, '', 0, None, "server or client host (ip or name)"],
    'port': [int, 0, 0, None, "server or client port"],
    'ports': [tuple, (), 0, None, "multi ports"],
    'use_multi_ports': [bool, False, 0, None, "use multi ports"],
    'ssl_options': [dict, {}, 0, None, "ssl options"],
    'tx_buffer_size': [int, 4096, 0, None, ""],
    'connected_event_name': [
        str, 'EV_CONNECTED', 0, None,
        "Must be empty if you don't want receive this event"
    ],
    'disconnected_event_name': [
        str, 'EV_DISCONNECTED', 0, None,
        "Must be empty if you don't want receive this event"
    ],
    'transmit_ready_event_name': [
        str, 'EV_TRANSMIT_READY', 0, None,
        "Must be empty if you don't want receive this event"
    ],
    'rx_data_event_name': [str, 'EV_RX_DATA', 0, None, ""],
    '_socket_map': [
        None, {}, GConfig.FLAG_DIRECT_ATTR, None,
        "Set by gaplic. Dict {fd:Gobj}"
    ],
    '_impl_poll': [
        None, None, GConfig.FLAG_DIRECT_ATTR, None,
        "Set by gaplic. epoll implementation"
    ],
    'addr': [
        None, None, GConfig.FLAG_DIRECT_ATTR, None,
        "Peername"
    ],

    # send_bytes is the number of bytes to send to socket.send().  Multiples
    # of 9000 should avoid partly-filled packets, but don't set this larger
    # than the TCP write buffer size.  In Linux, /proc/sys/net/ipv4/tcp_wmem
    # controls the minimum, default, and maximum sizes of TCP write buffers.
    'send_bytes': [int, 18000, 0, None, ""],

    # A tempfile should be created if the pending output is larger than
    # outbuf_overflow, which is measured in bytes. The default is 1MB.  This
    # is conservative.
    'outbuf_overflow': [int, 1048576, 0, None, ""],
    'close_when_flushed': [
        bool, False, 0, None,
        "True to close the socket when flushed"
    ],
    'n_gsocks': [int, 0, 0, None, "Server Stats: Number of gsocks opened"],
}


class GSock(GObj):
    """  Client socket gobj.

    Manage socket events.

    .. ginsfsm::
       :fsm: GSOCK_FSM
       :gconfig: GSOCK_GCONFIG


    *Input-Events:*
        * :attr:`'EV_SEND_DATA'`:

          Write data in the output data buffer and flush it right now.

          Equivalent to EV_WRITE_OUTPUT_DATA and EV_FLUSH_OUTPUT_DATA together.

          Event attributes:

            * ``data``: data to send.

        * :attr:`'EV_WRITE_OUTPUT_DATA'`:

          Write data in the output data buffer.

          Event attributes:

            * ``data``: data to send.

        * :attr:`'EV_FLUSH_OUTPUT_DATA'`:

          Flush the output buffer data.



    *Output-Events:*
        * :attr:`'EV_CONNECTED'`: socket connected.

          Attributes added to the sent :term:`event`:

            * ``peername``: remote address to which the socket is connected.
            * ``sockname``: the socket’s own address.

        * :attr:`'EV_DISCONNECTED'`: socket disconnected.

        * :attr:`'EV_TRANSMIT_READY'`: socket ready to transmit more data.

        * :attr:`'EV_RX_DATA'`: data received.

          The data is not buffered.
          As much data are read by recv() as data are broadcast.

          Event attributes:

            * ``data``: Data received from remote address.
    """
    _global_trace_dump = False

    def __init__(self):
        GObj.__init__(self, GSOCK_FSM, GSOCK_GCONFIG)

        self.txed_msgs = 0  # TODO:  estos alarmas/estadisticas
        self.rxed_msgs = 0
        self.txed_bytes = 0
        self.rxed_bytes = 0
        self.__trace_dump = False

        self.socket = None
        self.connected = False
        self.addr = None  # remote address to which the socket is connected
        self.sockname = None  # the socket’s own address

        self._socket_map = {}   # socket_map set by gaplic. Dict {fd:Gobj}
        self._impl_poll = None  # _poll(),epoll() implementation
        self._accepting = False
        self._transmit_ready_event_done = False
        self._clisrv = False
        self._fileno = None
        self.last_activity = 0  # Time of last activity
        self.will_close = False  # set to True to close the socket.
        # TODO: WARNING! outbufs directly used by c_http_clisrv!!
        self.outbufs = [OverflowableBuffer(self.config.outbuf_overflow)]

    def start_up(self):
        """ Initialization zone.

        Subcribe all enabled :term:`output-event`'s to ``subscriber``.
        By default the subscriber is the parent.
        """
        if self.config.subscriber is None:
            self.config.subscriber = self.parent
        self.subscribe_event(None, self.config.subscriber)

        # Verify the SSL options. Otherwise we don't get errors until clients
        # connect. This doesn't verify that the keys are legitimate, but
        # the SSL module doesn't do that until there is a connected socket
        # which seems like too much work
        if self.config.ssl_options:
            # Only certfile is required: it can contain both keys
            if 'certfile' not in self.config.ssl_options:
                raise KeyError('missing key "certfile" in ssl_options')

            if not os.path.exists(self.config.ssl_options['certfile']):
                raise ValueError(
                    'certfile "%s" does not exist' %
                    self.config.ssl_options['certfile'])
            if ('keyfile' in self.config.ssl_options and
                    not os.path.exists(self.config.ssl_options['keyfile'])):
                raise ValueError(
                    'keyfile "%s" does not exist' %
                    self.config.ssl_options['keyfile'])

    def go_out(self):
        """ Finish zone.
        """
        if self.connected:
            self.mt_drop()
        if self._fileno:
            self.remove_socket()

    def set_clisrv_socket(self, sock):
        # Set to nonblocking just to make sure for cases where we
        # get a socket from a blocking source.
        if hasattr(sock, 'setblocking'):
            # can be stdin
            sock.setblocking(0)
        self.add_socket(sock)
        self.connected = True
        self._clisrv = True
        # The constructor no longer requires that the socket
        # passed be connected.
        if hasattr(sock, 'getpeername'):
            # can be stdin
            try:
                self.addr = sock.getpeername()
            except socket.error as err:
                if err.args[0] == ENOTCONN:
                    # To handle the case where we got an unconnected
                    # socket.
                    self.connected = False
                else:
                    # The socket is broken in some unknown way, alert
                    # the user and remove it from the socket_map (to prevent
                    # polling of broken sockets).
                    self.remove_socket()
                    #raise
        if hasattr(sock, 'getsockname'):
            # can be stdin
            self.sockname = sock.getsockname()

    def create_socket(self, family, xtype):
        self.family_and_type = family, xtype
        sock = socket.socket(family, xtype)
        sock.setblocking(0)
        self.add_socket(sock)

    def set_reuse_addr(self):
        # try to re-use a server port if possible
        try:
            self.socket.setsockopt(
                socket.SOL_SOCKET, socket.SO_REUSEADDR,
                self.socket.getsockopt(socket.SOL_SOCKET,
                                       socket.SO_REUSEADDR) | 1
            )
        except socket.error:
            pass

    def add_socket(self, sock):
        self.socket = sock
        self._fileno = sock.fileno()
        self._socket_map[self._fileno] = self
        self._impl_poll.register(
            self._fileno, IOLoop.WRITE | IOLoop.READ | IOLoop.ERROR)

    def is_closed(self):
        return not self.socket

    def is_disconnected(self):
        return not self.connected

    def get_peername(self):
        """ Remote address to which the socket is connected
        """
        return self.addr

    def get_sockname(self):
        """ the socket’s own address
        """
        return self.sockname

    def remove_socket(self):
        fd = self._fileno
        self._socket_map.pop(fd, None)
        self._fileno = None
        self.socket = None
        try:
            self._impl_poll.unregister(fd)
        except (OSError, IOError):
            self.logger.error("Error deleting fd from IOLoop", exc_info=True)

    #==================================================
    #   socket object methods.
    #==================================================
    def listen(self, num):
        self._accepting = True
        if os.name == 'nt' and num > 5:
            num = 5
        return self.socket.listen(num)

    def bind(self, addr):
        self.addr = addr
        return self.socket.bind(addr)

    def connect(self, address):
        self.connected = False
        err = self.socket.connect_ex(address)
        if err in (EINPROGRESS, EALREADY, EWOULDBLOCK, ) \
                or err == EINVAL and os.name in ('nt', 'ce'):
            return
        if err in (0, EISCONN):
            self.addr = address
            self.handle_connect_event()
        else:
            raise socket.error(err, errorcode[err])

    def accept(self):
        # XXX can return either an address pair or None
        try:
            conn, addr = self.socket.accept()
        except TypeError:
            return None
        except socket.error as why:
            if why.args[0] in (EWOULDBLOCK, ECONNABORTED):
                return None
            else:
                raise
        else:
            return conn, addr

    def send(self, data):
        try:
            result = self.socket.send(data)
            if result == 0:
                # With OpenSSL, if we couldn't write the entire buffer,
                # the very same string object must be used on the
                # next call to send.  Therefore we suppress
                # merging the write buffer after an incomplete send.
                # A cleaner solution would be to set
                # SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER, but this is
                # not yet accessible from python
                # (http://bugs.python.org/issue8240)
                pass
            return result
        except socket.error as why:
            if why.args[0] == EWOULDBLOCK:
                return 0
            elif why.args[0] in _DISCONNECTED:
                self.handle_close()
                return 0
            else:
                raise

    def recv(self):
        try:
            data = self.socket.recv(8192)
            if not data:
                # a closed connection is indicated by signaling
                # a read condition, and having recv() return 0.
                self.handle_close()
                return b''
            else:
                return data
        except socket.error as why:
            # winsock sometimes throws ENOTCONN
            if why.args[0] in _DISCONNECTED:
                self.handle_close()
                return b''
            else:
                raise

    def close(self):
        self.connected = False
        self._accepting = False
        if self.socket:
            try:
                self.socket.close()
            except socket.error as why:
                if why.args[0] not in (ENOTCONN, EBADF):
                    raise

    # cheap inheritance, used to pass all other attribute
    # references to the underlying socket object.
    def __getattr__(self, attr):
        try:
            #retattr = getattr(self.socket, attr)
            retattr = socket.__getattr__(self.socket, attr)

        except AttributeError:
            raise AttributeError("%s instance has no attribute '%s'"
                                 % (self.__class__.__name__, attr))
        else:
            msg = "%(me)s.%(attr)s is deprecated; use %(me)s.socket.%(attr)s "\
                "instead" % {'me': self.__class__.__name__, 'attr': attr}
            warnings.warn(msg, DeprecationWarning, stacklevel=2)
            return retattr

    def handle_read_event(self):
        #self.logger.debug("handle_read_event    (%s %s)" % (self, self.name))
        if self._accepting:
            # _accepting sockets are never connected, they "spawn" new
            # sockets that are connected
            self.handle_accept()
        elif not self.connected:
            self.handle_connect_event()
            self.handle_read()
        else:
            self.handle_read()

    def handle_connect_event(self):
        #self.logger.debug("handle_connect_event (%s %s)" % (self, self.name))
        err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if err != 0:
            raise socket.error(err, _strerror(err))
        self.connected = True
        self.handle_connect()

    def handle_write_event(self):
        #self.logger.debug("handle_write_event   (%s %s)" % (self, self.name))
        if self._accepting:
            # Accepting sockets shouldn't get a write event.
            # We will pretend it didn't happen.
            return

        if not self.connected:
            #check for errors
            err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
            if err != 0:
                raise socket.error(err, _strerror(err))

            self.handle_connect_event()
        self.handle_write()

    def handle_error(self):
        nil, t, v, tbinfo = compact_traceback()

        # sometimes a user repr method will crash.
        try:
            self_repr = repr(self)
        except:
            self_repr = '<__repr__(self) failed for object at %0x>' % id(self)

        self.logger.error(
            'uncaptured python exception, closing channel %s (%s:%s %s)' % (
                self_repr,
                t,
                v,
                tbinfo
            )
        )
        self.handle_close()

    #====================================================
    #   Mine's
    #====================================================
    def handle_accept(self):
        try:
            pair = self.accept()
            if pair is not None:
                self.handle_accepted(*pair)

        except socket.error:
            # Linux: On rare occasions we get a bogus socket back from
            # accept. socketmodule.c:makesockaddr complains that the
            # address family is unknown. We don't want the whole server
            # to shut down because of this.
            self.logger.warning(
                'server accept() threw an exception',
                exc_info=True)

    def handle_accepted(self, sock, addr):
        sock.close()
        self.logger.warning('unhandled accepted event')

    def _mt_connect(self, **kw):
        """ Try to connect to (host, port).

        :param kw: valid keyword arguments are ``host`` and ``port``.

        This method calls to :meth:`get_next_dst` to get the destination tuple.

        Example::

            # override host,port attributes
            mt_connect(host='127.0.0.1', port=80)

        Or::

            # use current host,port attributes
            mt_connect()

        :class:`GSock` will broadcast some of ``'EV_CONNECTED'`` or
        ``'EV_DISCONNECTED'`` event as result of this action.
        """
        self.__dict__.update(**kw)

        if self.connected:
            self.logger.error(
                "ERROR connecting to host %r, port %r. "
                "ALREADY CONNECTED." % (self.config.host, self.config.port))
            return False
        if self.socket:
            self.logger.error(
                "ERROR connecting to host %r, port %r. SOCKET EXISTS." %
                (self.config.host, self.config.port))
            self.close()  # ???
            self.remove_socket()  # ???
            #TODO: si cumple el timeout de conexion viene por aqui
            #return False ???

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.config.host, self.config.port = self.get_next_dst()
        try:
            self.config.ip = ip = socket.gethostbyname(self.config.host)
        except Exception as e:
            self.logger.error('ERROR gethostbyname(%r) %s' % (
                self.config.host, e))
            return False
        hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.logger.info(
            "%s * %r ==> Connecting to host %r, ip %r, port %r..." % (
            hora,
            self,
            self.config.host,
            ip,
            self.config.port)
        )
        try:
            self.connect((ip, self.config.port))
        except Exception as e:
            self.logger.error("_mt_connect() ERROR %r" % e)
            return False
        return True

    def get_next_dst(self):
        """ Supply the destination ``(host,port)`` tuple to
        :meth:`_mt_connect` method.
        By default this function returns the internal ``(host,port)``
        attributes.

        TO BE OVERRIDE if you need other policy.
        """
        return (self.config.host, self.config.port)

    def handle_connect(self):
        self.addr = self.socket.getpeername()
        self.sockname = self.socket.getsockname()

        hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.logger.info("%s - %r <== Connected! %s host %s" % (
            hora,
            self,
            "FROM" if self._clisrv else "TO",
            str(self.addr)))
        # remove WRITE
        self._impl_poll.modify(self._fileno, IOLoop.READ | IOLoop.ERROR)
        if self.config.connected_event_name is not None:
            self.broadcast_event(
                self.config.connected_event_name,
                gsock=self,
                sockname=self.sockname,
                peername=self.addr,
            )

    def mt_drop(self):
        """ Drop the connexion.
        """
        hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.logger.info(
            "%s * %r ==> Drop from host %r, %r ..." % (
                hora,
                self,
                self.config.host,
                self.addr
            )
        )
        if not self.connected:
            self.logger.info("mt_drop(): Socket NOT connected!!")
            return 0
        if self.socket:
            try:
                self.socket.shutdown(socket.SHUT_RDWR)
                self._impl_poll.modify(
                    self._fileno, IOLoop.WRITE | IOLoop.READ | IOLoop.ERROR)
            except:
                self.close()
        else:
            self.close()
        return 0

    def handle_close(self):
        for outbuf in self.outbufs:
            try:
                outbuf._close()
            except:
                hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                self.logger.exception(
                    '%s - ERROR in %s: Unknown exception'
                    'while trying to close outbuf' % (hora, self.name))
        self.close()
        self.remove_socket()
        if 1:
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.logger.info(
                "%s * %r <== Disconnected! from %r %r %r" % (
                hora,
                self,
                self.config.host,
                self.config.port,
                self.addr)
            )
        #TODO: pon la causa del disconnect
        if self.config.disconnected_event_name is not None:
            self.broadcast_event(
                self.config.disconnected_event_name,
                gsock=self,
            )
        self.addr = None

    def handle_read(self):
        try:
            if hasattr(self.socket, "read"):
                # can be stdin
                data = self.socket.read()
            else:
                data = self.recv()
        except socket.error:
            self.handle_error()
            return

        ln = len(data)
        self.rxed_msgs += 1
        self.rxed_bytes += ln
        if self.trace_dump:
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.logger.info(
                "%s - Recv data %r (%d bytes)\n%s" % (
                    hora,
                    self,
                    ln,
                    hexdump('<==', data)
                )
            )

        if self.config.rx_data_event_name is not None:
            try:
                self.broadcast_event(
                    self.config.rx_data_event_name,
                    gsock=self,
                    data=data,
                )
            except:
                traceback.format_exc()
                hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                self.logger.error(
                    "%s - ERROR processing recv data %r (%d bytes)\n%s" % (
                        hora,
                        self,
                        ln,
                        hexdump('<==', data)
                    )
                )
                self.logger.error(traceback.format_exc())

    def readable(self):
        #"predicate for inclusion in the readable for select()"
        return True

    #-------------------------------------------------#
    #               Write zone
    #-------------------------------------------------#
    def any_outbuf_has_data(self):
        for outbuf in self.outbufs:
            if bool(outbuf):
                return True
        return False

    def writable(self):
        #"predicate for inclusion in the writable for select()"
        return self.any_outbuf_has_data() or \
            (not self.connected) or \
            self.will_close

    def mt_send_data(self, data, callback=None):
        """ Send data right now to the network.
        :param data: data to send.
        """
        self.mt_write(data)
        self.mt_flush(callback)

    def mt_write(self, data):
        """ Write in the output data buffer.
            Equivalent to waitress' write_soon().
            The data are not really sent until _send_some() is called.
        """
        if isinstance(data, text_type):
            data = data.encode(encoding='latin-1', errors='strict')
        if data:
            if data.__class__ is ReadOnlyFileBasedBuffer:
                # they used wsgi.file_wrapper
                self.outbufs.append(data)
                nextbuf = OverflowableBuffer(self.config.outbuf_overflow)
                self.outbufs.append(nextbuf)
            else:
                self.outbufs[-1].append(data)
            return len(data)
        return 0

    def mt_flush(self, callback=None):
        """ Send output buffer data to the network
        """
        if not self.connected or not self.socket:
            self.logger.error(
                "ERROR %r trying mt_flush but DISCONNECTED" % self)
            return

        # TODO: callback when TRANSMIT_READY
        self._transmit_ready_event_done = False
        self._send_some()

    def handle_write(self):
        # Precondition: there's data in the out buffer to be sent, or
        # there's a pending will_close request
        if not self.connected:
            # we dont want to close the channel twice
            return

        something_sent = self._send_some()
        if not self.connected:
            # can be disconnected after _send_some()
            return
        if not something_sent:
            if not self._transmit_ready_event_done:
                self._transmit_ready_event_done = True
                if self.config.transmit_ready_event_name is not None:
                    self.broadcast_event(
                        self.config.transmit_ready_event_name,
                        gsock=self,
                    )

    def _send_some(self):
        """ Send as much output data as possible.
        """
        sent = False
        try:
            sent = self._flush_some()
        except socket.error:
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.logger.exception('%s - ERROR in %s: Socket error' % (
                hora, self.name))
            self.will_close = True
        except:
            hora = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            self.logger.exception(
                '%s - ERROR in %s: Unexpected exception when flushing' % (
                    hora, self.name))
            self.will_close = True

        if self.config.close_when_flushed and not self.any_outbuf_has_data():
            self.config.close_when_flushed = False
            self.will_close = True

        if self.will_close:
            self.handle_close()
        return sent

    def _flush_some(self):
        # Send as much data as possible to our client
        sent = 0
        dobreak = False

        while True:
            outbuf = self.outbufs[0]
            outbuflen = len(outbuf)
            if outbuflen <= 0:
                # self.outbufs[-1] must always be a writable outbuf
                if len(self.outbufs) > 1:
                    toclose = self.outbufs.pop(0)
                    try:
                        toclose._close()
                    except:
                        self.logger.exception(
                            'Unexpected error when closing an outbuf')
                    continue  # pragma: no cover (coverage bug, it is hit)
                else:
                    dobreak = True

            while outbuflen > 0:
                chunk = outbuf.get(self.config.send_bytes)
                if self.trace_dump:
                    hora = datetime.datetime.now().strftime(
                        "%Y-%m-%d %H:%M:%S")
                    self.logger.info(
                        "%s - Send data %r (%d bytes)\n%s" % (
                            hora,
                            self,
                            len(chunk),
                            hexdump('==>', chunk)
                        )
                    )
                num_sent = self.send(chunk)
                if num_sent:
                    outbuf.skip(num_sent, True)
                    outbuflen -= num_sent
                    sent += num_sent
                else:
                    dobreak = True
                    break

            if dobreak:
                break

        if sent:
            self.last_activity = time.time()
            return True

        return False

    #-------------------------------------------------#
    #               trace zone
    #-------------------------------------------------#
    @staticmethod
    def global_trace_dump(value):
        value = True if value else False
        GSock._global_trace_dump = value

    @property
    def trace_dump(self):
        return self.__trace_dump or self._global_trace_dump

    @trace_dump.setter
    def trace_dump(self, value):
        """  Enable/Disable dump trace.
        """
        self.__trace_dump = value


class GSSLSock(GSock):
    """A GSock class to write to and read from a non-blocking SSL socket.

    If the socket passed to the constructor is already connected,
    it should be wrapped with::

        ssl.wrap_socket(sock, do_handshake_on_connect=False, **kwargs)

    Unconnected sockets will be wrapped when connect is finished.
    """

    def __init__(self):
        GSock.__init__(self)

    def start_up(self):
        """ Initialization zone.
        """
        self._ssl_accepting = True
        self._handshake_reading = False
        self._handshake_writing = False
        self._ssl_connect_callback = None

    def writable(self):
        #"predicate for inclusion in the writable for select()"
        return self._handshake_reading or super(GSSLSock, self).reading()

    def readable(self):
        #"predicate for inclusion in the readable for select()"
        return self._handshake_writing or super(GSSLSock, self).writing()

    def _do_ssl_handshake(self):
        # Based on code from test_ssl.py in the python stdlib
        try:
            self._handshake_reading = False
            self._handshake_writing = False
            self.socket.do_handshake()
        except ssl.SSLError as err:
            if err.args[0] == ssl.SSL_ERROR_WANT_READ:
                self._handshake_reading = True
                return
            elif err.args[0] == ssl.SSL_ERROR_WANT_WRITE:
                self._handshake_writing = True
                return
            elif err.args[0] in (ssl.SSL_ERROR_EOF,
                                 ssl.SSL_ERROR_ZERO_RETURN):
                return self.close()
            elif err.args[0] == ssl.SSL_ERROR_SSL:
                try:
                    peer = self.socket.getpeername()
                except:
                    peer = '(not connected)'
                self.logger.warning(
                    "SSL Error on %d %s: %s",
                    self.socket.fileno(), peer, err)
                return self.close()
            raise
        except socket.error as err:
            if err.args[0] in (errno.ECONNABORTED, errno.ECONNRESET):
                return self.close()
        else:
            self._ssl_accepting = False
            if self._ssl_connect_callback is not None:
                callback = self._ssl_connect_callback
                self._ssl_connect_callback = None
                self._run_callback(callback)

    def handle_read(self):
        if self._ssl_accepting:
            self._do_ssl_handshake()
            return
        super(GSSLSock, self).handle_read()

    def handle_write(self):
        if self._ssl_accepting:
            self._do_ssl_handshake()
            return
        super(GSSLSock, self).handle_write()

    def handle_connect(self):
        # When the connection is complete, wrap the socket for SSL
        # traffic.  Note that we do this by overriding _handle_connect
        # instead of by passing a callback to super().connect because
        # user callbacks are enqueued asynchronously on the IOLoop,
        # but since _handle_events calls _handle_connect immediately
        # followed by _handle_write we need this to be synchronous.
        self.socket = ssl.wrap_socket(self.socket,
                                      do_handshake_on_connect=False,
                                      **self.config.ssl_options)
        super(GSSLSock, self).handle_connect()

    # TODO para revisar

    def recv(self):
        """ TODO: pending to include
        while True:
            # Read from the socket until we get EWOULDBLOCK or equivalent.
            # SSL sockets do some internal buffering, and if the data is
            # sitting in the SSL object's buffer select() and friends
            # can't see it; the only way to find out if it's there is to
            # try to read it.
            if not super(GSSLSock, self).recv():
                break
        """

        if self._ssl_accepting:
            # If the handshake hasn't finished yet, there can't be anything
            # to read (attempting to read may or may not raise an exception
            # depending on the SSL version)
            return b''

        try:
            # SSLSocket objects have both a read() and recv() method,
            # while regular sockets only have recv().
            # The recv() method blocks (at least in python 2.6) if it is
            # called when there is nothing to read, so we have to use
            # read() instead.
            data = self.socket.read(8192)
            if not data:
                # a closed connection is indicated by signaling
                # a read condition, and having recv() return 0.
                self.handle_close()
                return b''
            else:
                return data
        except ssl.SSLError as why:
            # SSLError is a subclass of socket.error, so this except
            # block must come first.
            if why.args[0] == ssl.SSL_ERROR_WANT_READ:
                return b''
            else:
                raise
        except socket.error as why:
            # winsock sometimes throws ENOTCONN
            if why.args[0] in _DISCONNECTED:
                self.handle_close()
                return b''
            elif why.args[0] in (EWOULDBLOCK, EAGAIN):
                return b''
            else:
                raise


# ---------------------------------------------------------------------------
# used for debugging.
# ---------------------------------------------------------------------------
def compact_traceback():
    t, v, tb = sys.exc_info()
    tbinfo = []
    if not tb:  # Must have a traceback
        raise AssertionError("traceback does not exist")
    while tb:
        tbinfo.append((
            tb.tb_frame.f_code.co_filename,
            tb.tb_frame.f_code.co_name,
            str(tb.tb_lineno)
        ))
        tb = tb.tb_next

    # just to be safe
    del tb

    file, function, line = tbinfo[-1]
    info = ' '.join(['[%s|%s|%s]' % x for x in tbinfo])
    return (file, function, line), t, v, info


def close_all_sockets(socket_map, ignore_all=False):
    for x in list(socket_map.values()):
        try:
            x.close()
        except OSError as x:
            if x.args[0] == EBADF:
                pass
            elif not ignore_all:
                raise
        except _reraised_exceptions:
            raise
        except:
            if not ignore_all:
                raise
    socket_map.clear()


#================================================================
#   Poll implementations.
#   Code copied from tornado/ioloop.py
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#================================================================
class _EPoll(object):
    """An epoll-based event loop using our C module for Python 2.5 systems"""
    _EPOLL_CTL_ADD = 1
    _EPOLL_CTL_DEL = 2
    _EPOLL_CTL_MOD = 3

    def __init__(self):
        self._epoll_fd = epoll.epoll_create()

    def fileno(self):
        return self._epoll_fd

    def close(self):
        os.close(self._epoll_fd)

    def register(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_ADD, fd, events)

    def modify(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_MOD, fd, events)

    def unregister(self, fd):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_DEL, fd, 0)

    def poll(self, timeout):
        return epoll.epoll_wait(self._epoll_fd, int(timeout * 1000))


class _KQueue(object):
    """A kqueue-based event loop for BSD/Mac systems."""
    def __init__(self):
        self._kqueue = select.kqueue()
        self._active = {}

    def fileno(self):
        return self._kqueue.fileno()

    def close(self):
        self._kqueue.close()

    def register(self, fd, events):
        if fd in self._active:
            raise IOError("fd %d already registered" % fd)
        self._control(fd, events, select.KQ_EV_ADD)
        self._active[fd] = events

    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        events = self._active.pop(fd)
        self._control(fd, events, select.KQ_EV_DELETE)

    def _control(self, fd, events, flags):
        kevents = []
        if events & IOLoop.WRITE:
            kevents.append(select.kevent(
                fd, filter=select.KQ_FILTER_WRITE, flags=flags))
        if events & IOLoop.READ or not kevents:
            # Always read when there is not a write
            kevents.append(select.kevent(
                fd, filter=select.KQ_FILTER_READ, flags=flags))
        # Even though control() takes a list, it seems to return EINVAL
        # on Mac OS X (10.6) when there is more than one event in the list.
        for kevent in kevents:
            self._kqueue.control([kevent], 0)

    def poll(self, timeout):
        kevents = self._kqueue.control(None, 1000, timeout)
        events = {}
        for kevent in kevents:
            fd = kevent.ident
            if kevent.filter == select.KQ_FILTER_READ:
                events[fd] = events.get(fd, 0) | IOLoop.READ
            if kevent.filter == select.KQ_FILTER_WRITE:
                if kevent.flags & select.KQ_EV_EOF:
                    # If an asynchronous connection is refused, kqueue
                    # returns a write event with the EOF flag set.
                    # Turn this into an error for consistency with the
                    # other IOLoop implementations.
                    # Note that for read events, EOF may be returned before
                    # all data has been consumed from the socket buffer,
                    # so we only check for EOF on write events.
                    events[fd] = IOLoop.ERROR
                else:
                    events[fd] = events.get(fd, 0) | IOLoop.WRITE
            if kevent.flags & select.KQ_EV_ERROR:
                events[fd] = events.get(fd, 0) | IOLoop.ERROR
        return events.items()


class _Select(object):
    """A simple, select()-based IOLoop implementation for non-Linux systems"""
    def __init__(self):
        self.read_fds = set()
        self.write_fds = set()
        self.error_fds = set()
        self.fd_sets = (self.read_fds, self.write_fds, self.error_fds)

    def close(self):
        pass

    def register(self, fd, events):
        if fd in self.read_fds or fd in self.write_fds or fd in self.error_fds:
            raise IOError("fd %d already registered" % fd)
        if events & IOLoop.READ:
            self.read_fds.add(fd)
        if events & IOLoop.WRITE:
            self.write_fds.add(fd)
        if events & IOLoop.ERROR:
            self.error_fds.add(fd)
            # Closed connections are reported as errors by epoll and kqueue,
            # but as zero-byte reads by select, so when errors are requested
            # we need to listen for both read and error.
            self.read_fds.add(fd)

    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        self.read_fds.discard(fd)
        self.write_fds.discard(fd)
        self.error_fds.discard(fd)

    def poll(self, timeout):
        readable, writeable, errors = select.select(
            self.read_fds, self.write_fds, self.error_fds, timeout)
        events = {}
        for fd in readable:
            events[fd] = events.get(fd, 0) | IOLoop.READ
        for fd in writeable:
            events[fd] = events.get(fd, 0) | IOLoop.WRITE
        for fd in errors:
            events[fd] = events.get(fd, 0) | IOLoop.ERROR
        return events.items()

#================================================================
#   Choose a poll implementation. Use epoll if it is available,
#   fall back to select() for non-Linux platforms
#================================================================
if hasattr(select, "epoll"):
    # Python 2.6+ on Linux
    _poll = select.epoll
elif hasattr(select, "kqueue"):
    # Python 2.6+ on BSD or Mac
    _poll = _KQueue
else:
    try:
        # Linux systems with our C module installed
        from ginsfsm import epoll
        _poll = _EPoll
    except Exception:
        # All other systems
        if "linux" in sys.platform:
            print("epoll module not found; using select()")
        _poll = _Select



# -*- encoding: utf-8 -*-
""" GWebSocket GObj

Implementation of the WebSocket protocol.

`WebSockets <http://dev.w3.org/html5/websockets/>`_ allow for bidirectional
communication between the browser and server.

.. warning::

   The WebSocket protocol was recently finalized as `RFC 6455
   <http://tools.ietf.org/html/rfc6455>`_ and is not yet supported in
   all browsers.  Refer to http://caniuse.com/websockets for details
   on compatibility.

   This module only supports the latest version 13 of the RFC 6455 protocol.

.. autoclass:: GWebSocket
    :members: start_up

"""
import base64
import array
from os import urandom
import hashlib
import struct
import uuid

from ginsfsm import __version__
from ginsfsm.utils import hexdump
import ginsfsm.escape
from ginsfsm.gconfig import GConfig
from ginsfsm.gmsg import GMsg
from ginsfsm.gobj import GObj
from ginsfsm.c_timer import GTimer
from ginsfsm.compat import (
    bytes_,
)
from ginsfsm.circular_fifo import CircularFIFO
from ginsfsm.buffers import OverflowableBuffer
from ginsfsm.protocols.http.common.parser import (
    HTTPRequestParser,
    HTTPResponseParser,
    build_environment,
)

STRUCT_2BYTES = struct.Struct("!H")
STRUCT_8BYTES = struct.Struct("!Q")
STRUCT_BB = struct.Struct("BB")
STRUCT_BBH = struct.Struct("!BBH")
STRUCT_BBQ = struct.Struct("!BBQ")

OPCODE_CONTINUATION_FRAME = 0x0
OPCODE_TEXT_FRAME = 0x01
OPCODE_BINARY_FRAME = 0x02

OPCODE_CONTROL_CLOSE = 0x08
OPCODE_CONTROL_PING = 0x09
OPCODE_CONTROL_PONG = 0x0A

# websocket supported version.
VERSION = 13

# closing frame status codes.
STATUS_NORMAL = 1000
STATUS_GOING_AWAY = 1001
STATUS_PROTOCOL_ERROR = 1002
STATUS_UNSUPPORTED_DATA_TYPE = 1003
STATUS_STATUS_NOT_AVAILABLE = 1005
STATUS_ABNORMAL_CLOSED = 1006
STATUS_INVALID_PAYLOAD = 1007
STATUS_POLICY_VIOLATION = 1008
STATUS_MESSAGE_TOO_BIG = 1009
STATUS_INVALID_EXTENSION = 1010
STATUS_UNEXPECTED_CONDITION = 1011
STATUS_TLS_HANDSHAKE_ERROR = 1015


def _create_sec_websocket_key():
    uid = uuid.uuid4()
    return base64.encodestring(uid.bytes).strip()


class FrameHead(object):
    """ Websocket frame head.

    This class analize the first two bytes of the header.
    The maximum size of a frame header is 14 bytes.

      0                   1
      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
     +-+-+-+-+-------+-+-------------+
     |F|R|R|R| opcode|M| Payload len |
     |I|S|S|S|  (4)  |A|     (7)     |
     |N|V|V|V|       |S|             |
     | |1|2|3|       |K|             |
     +-+-+-+-+-------+-+-------------+

    Full header:

      0                   1                   2                   3
      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
     +-+-+-+-+-------+-+-------------+-------------------------------+
     |F|R|R|R| opcode|M| Payload len |    Extended payload length    |
     |I|S|S|S|  (4)  |A|     (7)     |             (16/64)           |
     |N|V|V|V|       |S|             |   (if payload len==126/127)   |
     | |1|2|3|       |K|             |                               |
     +-+-+-+-+-------+-+-------------+ - - - - - - - - - - - - - - - +
     |     Extended payload length continued, if payload len == 127  |
     + - - - - - - - - - - - - - - - +-------------------------------+
     |                               |Masking-key, if MASK set to 1  |
     +-------------------------------+-------------------------------+
     | Masking-key (continued)       |          Payload Data         |
     +-------------------------------- - - - - - - - - - - - - - - - +
     :                     Payload Data continued ...                :
     + - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - +
     |                     Payload Data continued ...                |
     +---------------------------------------------------------------+
    """

    packer = struct.Struct('BB')

    def __init__(self, circular):
        # Information of the first two bytes header
        self.h_fin = 0  # final fragment in a message
        self.h_reserved_bits = 0
        self.h_opcode = 0
        self.h_mask = 0  # Set to 1 a masking key is present
        self.h_payload_len = 0
        self.circular = circular
        self.busy = False  # in half of header
        self.header_complete = False  # Set True when header is completed
        self.error = False

    def prepare_new_frame(self):
        """Reset variables for a new read.
        """
        # state of frame
        self.busy = True  # in half of header
        self.header_complete = False  # Set True when header is completed
        self.error = False

        # must do
        self.must_read_2_extended_payload_length = False
        self.must_read_8_extended_payload_length = False
        self.must_read_masking_key = False
        self.must_read_payload_data = False

        # information of frame
        self.masking_key = 0
        self.frame_length = 0

    def decode_head(self, data):
        """ READ: Decode the two bytes head.
        """
        if len(data) != 2:
            raise RuntimeError('ERROR decode_ntoh_header needs 2 bytes')
        byte1, byte2 = self.packer.unpack(bytes(data))

        # decod byte1
        self.h_fin = byte1 & 0x80
        self.h_reserved_bits = byte1 & 0x70
        self.h_opcode = byte1 & 0x0f

        # decod byte2
        self.h_mask = byte2 & 0x80
        self.h_payload_len = byte2 & 0x7f

        # analize

        if self.h_mask:
            # must read 4 bytes of masking key
            self.must_read_masking_key = True

        ln = self.h_payload_len
        if ln == 0:
            pass  # no data to read
        else:
            self.must_read_payload_data = True
            if ln < 126:
                # Got all we need to read data
                self.frame_length = self.h_payload_len
            elif ln == 126:
                # must read 2 bytes of extended payload length
                self.must_read_2_extended_payload_length = True
            else:  # ln == 127:
                # must read 8 bytes of extended payload length
                self.must_read_8_extended_payload_length = True

    def decode_extended_length(self, data):
        """ READ: Decode 2/8 bytes of extended payload length.
        """
        if self.must_read_2_extended_payload_length:
            # Read 2 bytes of extended payload length
            if len(data) != 2:
                raise RuntimeError(
                    'ERROR decode_extended_length needs 2 bytes')
            self.frame_length = STRUCT_2BYTES.unpack(bytes(data))[0]
            self.must_read_2_extended_payload_length = False

        elif self.must_read_8_extended_payload_length:
            # Read 8 bytes of extended payload length
            if len(data) != 8:
                raise RuntimeError(
                    'ERROR decode_extended_length needs 8 bytes')
            self.frame_length = STRUCT_8BYTES.unpack(bytes(data))[0]
            self.must_read_8_extended_payload_length = False

    def decode_masking_key(self, data):
        """ READ: Decode the 4 bytes of masking key.
        """
        if len(data) != 4:
            raise RuntimeError('ERROR decode_masking_key needs 4 bytes')
        self.masking_key = data
        self.must_read_masking_key = False

    def consume(self, bf):
        """ Consume input data to get and analyze the frame header.
            Return the consumed size.
        """
        # better work with local variables: more rapid.
        circular = self.circular
        total_consumed = 0
        if not self.busy:
            # waiting the first two byte's head
            available = circular.busy_space + len(bf)
            if available < 2:
                # save the remaining data
                consumed = circular.putdata(bf)
                total_consumed += consumed
                return total_consumed  # wait more data

            needed = 2 - circular.busy_space
            if needed > 0:
                consumed = circular.putdata(bf, needed)
                bf = bf[consumed:]
                total_consumed += consumed

            # we've got enough data! Start a new frame
            self.prepare_new_frame()  # `busy` flag is set.
            data = circular.getdata(2)
            self.decode_head(data)

        # processing extended header
        if self.must_read_2_extended_payload_length:
            available = circular.busy_space + len(bf)
            if available < 2:
                # save the remaining data
                consumed = circular.putdata(bf)
                total_consumed += consumed
                return total_consumed  # wait more data

            needed = 2 - circular.busy_space
            if needed > 0:
                consumed = circular.putdata(bf, needed)
                bf = bf[consumed:]
                total_consumed += consumed

            data = circular.getdata(2)
            self.decode_extended_length(data)  # flag is set.

        if self.must_read_8_extended_payload_length:
            available = circular.busy_space + len(bf)
            if available < 8:
                # save the remaining data
                consumed = circular.putdata(bf)
                total_consumed += consumed
                return total_consumed  # wait more data

            needed = 8 - circular.busy_space
            if needed > 0:
                consumed = circular.putdata(bf, needed)
                bf = bf[consumed:]
                total_consumed += consumed

            data = circular.getdata(8)
            self.decode_extended_length(data)  # flag is set.

        if self.must_read_masking_key:
            available = circular.busy_space + len(bf)
            if available < 4:
                # save the remaining data
                consumed = circular.putdata(bf)
                total_consumed += consumed
                return total_consumed  # wait more data

            needed = 4 - circular.busy_space
            if needed > 0:
                consumed = circular.putdata(bf, needed)
                bf = bf[consumed:]
                total_consumed += consumed

            data = circular.getdata(4)
            self.decode_masking_key(data)  # flag is set

        self.header_complete = True
        return total_consumed

    @property
    def size(self):
        return self.packer.size


def analyze_header(gmsg, iam_server):
    """ Analyze and process the header.
    """
    hd = FrameHead()
    data = gmsg.getdata(hd.size)
    if data is None:
        # there is no enough data
        return hd
    hd.network_to_host(data)
    hd.complete = True
    if hd.h_reserved_bits:
        # client is using as-yet-undefined extensions; abort
        hd.error = True
        gmsg.logger and gmsg.logger.error(
            "ERROR websocket header: using as-yet-undefined h_reserved_bits")
        return hd

    if hd.opcode_is_control:
        #
        #   Control h_opcode
        #
        control = hd.h_opcode
        if control == OPCODE_CONTROL_CLOSE:
            hd.close = True
            pass
        elif control == OPCODE_CONTROL_PING:
            pass
        elif control == OPCODE_CONTROL_PONG:
            pass
        if hd.h_payload_len >= 126:
            # control frames must have payload < 126; abort
            hd.error = True
            gmsg.logger and gmsg.logger.error(
                "ERROR websocket header: too big control frame payloda")
    else:
        #
        #   Data h_opcode
        #
        if iam_server and not hd.h_mask:
            # Unmasked frame; abort
            hd.error = True
            gmsg.logger and gmsg.logger.error(
                "ERROR websocket header: iam_server but unmasked frame")

    return hd


#----------------------------------------------------#
#               Machine Actions
#----------------------------------------------------#
def ac_timeout_waiting_handshake(self, event):
    """ Too much time waiting the handshake.
    """
    self.logger and self.logger.error(
        "ERROR websocket TIMEOUT waiting HANDSHAKE: %r" % self)
    self.on_close_broadcasted = True  # no on_open was broadcasted
    self.close()


def ac_timeout_waiting_frame_header(self, event):
    """ Timeout frame start.
        Too much time waiting the frame header.
    """
    self.start_inactivity_timer(self.config.ping_interval)
    self.ping()


def ac_timeout_waiting_payload_data(self, event):
    """ Timeout frame start.
        Too much time waiting the frame header.
    """
    self.logger and self.logger.error(
        "ERROR websocket TIMEOUT waiting PAYLOAD data")
    self.close()


def ac_connected(self, event):
    """ iam client. send the request
    """
    # send the request and wait the response
    host = self.gsock.host
    port = self.gsock.port
    resource = self.config.resource
    self.do_request(host, port, resource)


def ac_disconnected(self, event):
    """ Partner has disconnected.
    """
    self.closed = True
    self.close()


def ac_process_handshake(self, event):
    """ Process handshake rx data.
    We can be server or client.
    """
    data = event.data
    if self.config.iam_server:
        # analyze the request and response it
        request = self._process_request(data)
        if request:
            self.request = request
            ok = self.do_response()
            if not ok:
                # request refused! Drop connection.
                self.post_event(self.gsock, 'EV_DROP')
                return
            #------------------------------------#
            #   Upgrade to websocket
            #------------------------------------#
            self.set_new_state('ST_WAITING_HEADER')
            self.gaplic.add_callback(self.broadcast_event, 'EV_ON_OPEN')

    else:
        # analyze the response
        response = self._process_response(data)
        if response:
            if response.status == 101:
                #------------------------------------#
                #   Upgrade to websocket
                #------------------------------------#
                self.set_new_state('ST_WAITING_HEADER')
                self.gaplic.add_callback(self.broadcast_event, 'EV_ON_OPEN')
            else:
                self.logger.error(
                    "ERROR websocket Response %d, waiting 101",
                    response.status)
                self.logger.error(hexdump('<==', data))
                self.close()


def ac_process_frame_header(self, event):
    """ Processing the header.
    """
    self.start_inactivity_timer(self.config.ping_interval)
    cur_frame = self.cur_frame
    bf = event.data
    while bf:
        consumed = cur_frame.consume(bf)
        if cur_frame.error:
            self.close()  # on error do break the connection
            return
        bf = bf[consumed:]

        if cur_frame.header_complete:
            if cur_frame.must_read_payload_data:
                # Creat a new buffer for payload data
                self.loaded_data = 0
                self.set_new_state('ST_WAITING_PAYLOAD_DATA')
                self.send_event(self, 'EV_RX_DATA', data=bf)
                return
            else:
                self.frame_completed()


def ac_process_payload_data(self, event):
    """ Get payload data
    """
    bf = event.data
    bf_len = len(bf)
    if bf_len == 0:
        return
    loaded_data = self.loaded_data
    frame_length = self.cur_frame.frame_length
    to_add = frame_length - loaded_data
    if to_add < bf_len:
        # use partial buffer, remain must be another new frame
        data = bf[0:to_add]
        bf = bf[to_add:]
        self.frame_gmsg.putdata(data)
        loaded_data += to_add
    else:
        self.frame_gmsg.putdata(bf)
        loaded_data += bf_len
        bf = None

    self.loaded_data = loaded_data
    if loaded_data == frame_length:
        self.frame_completed()
        self.set_new_state('ST_WAITING_HEADER')
        if bf:
            self.send_event(self, 'EV_RX_DATA', data=bf)


def ac_send_json_message(self, event):
    msg = event.data
    self.send_message(msg)


GWEBSOCKET_FSM = {
    'event_list': (
        'EV_SEND_JSON_MESSAGE: top input',
        'EV_ON_OPEN:top output',
        'EV_ON_CLOSE:top output',
        'EV_ON_MESSAGE:top output',
        'EV_RX_DATA:bottom input',
        # 'EV_TRANSMIT_READY:bottom input',
        'EV_CONNECTED:bottom input',
        'EV_DISCONNECTED:bottom input',
        'EV_SET_TIMER:bottom output',
        'EV_TIMEOUT:bottom input',
    ),
    'state_list': (
        'ST_WAITING_HANDSHAKE',
        'ST_WAITING_HEADER',
        'ST_WAITING_PAYLOAD_DATA',
    ),
    'machine': {
        'ST_WAITING_HANDSHAKE':
        (
            ('EV_CONNECTED',    ac_connected,                       None),
            ('EV_DISCONNECTED', None,                               None),
            ('EV_TIMEOUT',      ac_timeout_waiting_handshake,       None),
            ('EV_RX_DATA',      ac_process_handshake,               None),
        ),

        'ST_WAITING_HEADER':
        (
            ('EV_SEND_JSON_MESSAGE', ac_send_json_message,          None),
            ('EV_DISCONNECTED', ac_disconnected,                    None),
            ('EV_TIMEOUT',      ac_timeout_waiting_frame_header,    None),
            ('EV_RX_DATA',      ac_process_frame_header,            None),
        ),

        'ST_WAITING_PAYLOAD_DATA':
        (
            ('EV_SEND_JSON_MESSAGE', ac_send_json_message,          None),
            ('EV_DISCONNECTED', ac_disconnected,                    None),
            ('EV_TIMEOUT',      ac_timeout_waiting_payload_data,    None),
            ('EV_RX_DATA',      ac_process_payload_data,            None),
        ),
    }
}

GWEBSOCKET_GCONFIG = {
    'gsock': [
        None, None, GConfig.FLAG_DIRECT_ATTR, None, "GSock connection."
    ],
    'request': [
        None, None, GConfig.FLAG_DIRECT_ATTR, None, "websocket request."],
    'iam_server': [bool, True, 0, None, "What side? server or client."],
    'resource': [str, '/', 0, None, "Resource when iam client."],
    'ping_interval': [int, 5*60, 0, None, "Ping interval"],

    # A tempfile should be created if the pending output is larger than
    # outbuf_overflow, which is measured in bytes. The default is 1MB.  This
    # is conservative.
    # It's too used for maximum size of GMsg (buffer for frame payload data)
    'outbuf_overflow': [int, 1048576, 0, None, ""],

    'subscriber': [
        None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
    ],
}


class GWebSocket(GObj):
    """  GWebSocket GObj.
    WebSocket protocol:
    `RFC 6455  <http://tools.ietf.org/html/rfc6455>`_.

    .. ginsfsm::
       :fsm: GWEBSOCKET_FSM
       :gconfig: GWEBSOCKET_GCONFIG

    *Input-Events:*
        * :attr:`'EV_INPUT_EVENT'`: sample input event.

          Event attributes:

              * ``data``: sample event attribute.

    *Output-Events:*
        * :attr:`'EV_OUTPUT_EVENT'`: sample output event.

          Event attributes:

              * ``data``: sample event attribute.

    """

    KEY = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"

    def __init__(self):
        """ I used Cicular FIFO buffer to save the received data.
            I pull the bytes needed for each operation.
            The buffer is fixed size and by default of 8K bytes,
            and it intend be as rapid as possible.
            It's enough for do analysis of the header.
            The payload data are saved in a OverflowableBuffer buffer,
            a high capacity buffer.
        """
        GObj.__init__(self, GWEBSOCKET_FSM, GWEBSOCKET_GCONFIG)
        self.circular = CircularFIFO(14)  # always the same.
        self.cur_frame = FrameHead(self.circular)  # always the same
        self.closed = False  # channel closed
        self.on_close_broadcasted = False  # event on_close already broadcasted
        self.message_header = None
        self.frame_gmsg = GMsg(1024, self.config.outbuf_overflow, self.logger)
        self.message_buffer = None
        self.parsing_request = None  # A request parser instance
        self.parsing_response = None  # A response parser instance
        self.ginsfsm_user_agent = False  # True when user-agent is ginsfsm
                                    # When user is a browser, it'll be sockjs.
                                    # when user is not browser, at the moment,
                                    # we only recognize ginsfsm websocket.

    def start_up(self):
        """ Initialization zone.
        We always have got the connection done (with gsock)
        because this gobj works:
            * with pyramid web environment
            * or own handshake.

        In own handshake, if iam a client, we initiate the request http.

        """
        if self.config.subscriber:
            self.subscribe_event(None, self.config.subscriber)

        if not self.gsock:
            raise RuntimeError('ERROR websocket needs a gsock done!')

        if self.request:
            # We've got already the request (pyramid environment)
            ok = self.do_response()
            if not ok:
                # request refused! Drop connection.
                self.post_event(self.gsock, 'EV_DROP')
                return
            #------------------------------------#
            #   Upgrade to websocket
            #------------------------------------#
            #   Subscribe all gsock events.
            #------------------------------------#
            self.gsock.delete_all_subscriptions()  # TODO borrando delete http?
            self.gsock.subscribe_event(None, self)
            # TODO: mira interactuar con los timers del canal http.
            self.set_new_state('ST_WAITING_HEADER')
            self.gaplic.add_callback(self.broadcast_event, 'EV_ON_OPEN')

        else:
            #------------------------------------#
            #   Own handshake.
            #   We can be server or client.
            #------------------------------------#
            #   Subscribe all gsock events.
            #------------------------------------#
            self.gsock.delete_all_subscriptions()  # TODO borrando delete http?
            self.gsock.subscribe_event(None, self)

            if self.config.iam_server:
                # wait the request
                pass
            else:
                # send the request and wait the response
                host = self.gsock.read_parameter('host')
                port = self.gsock.read_parameter('port')
                resource = self.read_parameter('resource')
                self.do_request(host, port, resource)

        # Setup the timers
        self.inactivity_timer = self.create_gobj(
            None,
            GTimer,
            self,
            timeout_event_name='EV_TIMEOUT'
        )
        self.start_inactivity_timer(5)

    def go_out(self):
        """ Finish zone.
        """

    def start_inactivity_timer(self, seconds):
        self.send_event(
            self.inactivity_timer,
            'EV_SET_TIMER',
            seconds=seconds
        )

    def stop_inactivity_timer(self):
        self.send_event(
            self.inactivity_timer,
            'EV_SET_TIMER',
            seconds=-1
        )

    def do_request(self, host, port, resource, **options):
        headers = []
        headers.append("GET %s HTTP/1.1" % resource)
        headers.append("User-Agent: ginsfsm-%s" % __version__)
        headers.append("Upgrade: websocket")
        headers.append("Connection: Upgrade")
        if port == 80:
            hostport = host
        else:
            hostport = "%s:%d" % (host, port)
        headers.append("Host: %s" % hostport)

        if "origin" in options:
            headers.append("Origin: %s" % options["origin"])
        else:
            headers.append("Origin: %s" % hostport)

        key = _create_sec_websocket_key()
        headers.append("Sec-WebSocket-Key: %s" % key)
        headers.append("Sec-WebSocket-Version: %s" % VERSION)
        if "header" in options:
            headers.extend(options["header"])

        headers.append("")
        headers.append("")

        header_str = "\r\n".join(headers)
        data = bytes_(header_str)

        self.send_event(self.gsock, 'EV_SEND_DATA', data=data)

    def do_response(self):
        """ Got the request: analyze it and send the response.
        """
        # Websocket only supports GET method
        if self.request.method != "GET":
            data = bytes_(
                "HTTP/1.1 405 Method Not Allowed\r\n"
                "Allow: GET\r\n"
                "Connection: Close\r\n"
                "\r\n"
            )
            self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            return False

        if hasattr(self.request, 'environ'):
            environ = self.request.environ
        else:
            environ = build_environment(self.request, '', '', '')

        user_agent = environ.get("HTTP_USER_AGENT", "").lower()
        ln = len("ginsfsm")
        if len(user_agent) >= ln:
            if user_agent[0:ln] == "ginsfsm":
                self.ginsfsm_user_agent = True

        # Upgrade header should be present and should be equal to WebSocket
        if environ.get("HTTP_UPGRADE", "").lower() != "websocket":
            data = bytes_(
                "HTTP/1.1 400 Bad Request\r\n"
                "Connection: Close\r\n"
                "\r\n"
                "Can \"Upgrade\" only to \"WebSocket\"."
            )
            self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            return False

        # Connection header should be upgrade.
        # Some proxy servers/load balancers
        # might mess with it.
        connection = list(
            map(
                lambda s: s.strip().lower(),
                environ.get("HTTP_CONNECTION", "").split(",")
            )
        )
        if "upgrade" not in connection:
            data = bytes_(
                "HTTP/1.1 400 Bad Request\r\n"
                "Connection: Close\r\n"
                "\r\n"
                "\"Connection\" must be \"Upgrade\"."
            )
            self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            return False

        # The difference between version 8 and 13 is that in 8 the
        # client sends a "Sec-Websocket-Origin" header and in 13 it's
        # simply "Origin".
        supported_version = ("7", "8", "13")
        if not environ.get(
                "HTTP_SEC_WEBSOCKET_VERSION") in supported_version:
            data = bytes_(
                "HTTP/1.1 426 Upgrade Required\r\n"
                "Sec-WebSocket-Version: 13\r\n\r\n"
            )
            self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            return False

        fields = (
            "HTTP_HOST",
            "HTTP_SEC_WEBSOCKET_KEY",
            "HTTP_SEC_WEBSOCKET_VERSION"
        )
        if not all(list(map(lambda f: environ.get(f), fields))):
            data = bytes_(
                "HTTP/1.1 400 Bad Request\r\n"
                "Connection: Close\r\n"
                "\r\n"
                "Missing/Invalid WebSocket headers."
            )
            self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            return False

        key = environ.get("HTTP_SEC_WEBSOCKET_KEY", None)
        if not key or len(base64.b64decode(bytes_(key))) != 16:
            data = bytes_(
                "HTTP/1.1 400 Bad Request\r\n"
                "Connection: Close\r\n"
                "\r\n"
                "Sec-Websocket-Key is invalid key."
            )
            # TODO: review
            #self.send_event(self.gsock, 'EV_SEND_DATA', data=data)
            #return False

        #------------------------------#
        #   Accept the connection
        #------------------------------#
        subprotocol_header = ''
        subprotocols = environ.get("HTTP_SEC_WEBSOCKET_PROTOCOL", '')
        subprotocols = [s.strip() for s in subprotocols.split(',')]
        if subprotocols:
            selected = self.select_subprotocol(subprotocols)
            if selected:
                assert selected in subprotocols
                subprotocol_header = "Sec-WebSocket-Protocol: %s\r\n" % (
                    selected)

        data = bytes_(
            "HTTP/1.1 101 Switching Protocols\r\n"
            "Upgrade: websocket\r\n"
            "Connection: Upgrade\r\n"
            "Sec-WebSocket-Accept: %s\r\n"
            "%s"
            "\r\n" % (
                self._challenge_response(key),
                subprotocol_header)
        )
        self.send_event(self.gsock, 'EV_SEND_DATA', data=data)

        return True

    @staticmethod
    def compute_accept_value(key):
        """Computes the value for the Sec-WebSocket-Accept header,
        given the value for Sec-WebSocket-Key.
        """
        sha1 = hashlib.sha1()
        sha1.update(bytes_(key))
        sha1.update(b"258EAFA5-E914-47DA-95CA-C5AB0DC85B11")  # Magic value
        return ginsfsm.escape.native_str(base64.b64encode(sha1.digest()))

    def _challenge_response(self, key):
        return GWebSocket.compute_accept_value(key)

    def select_subprotocol(self, subprotocols):
        """Invoked when a new WebSocket requests specific subprotocols.

        ``subprotocols`` is a list of strings identifying the
        subprotocols proposed by the client.  This method may be
        overridden to return one of those strings to select it, or
        ``None`` to not select a subprotocol.  Failure to select a
        subprotocol does not automatically abort the connection,
        although clients may close the connection if none of their
        proposed subprotocols was selected.
        """
        return None

    def close(self):
        if not self.closed:
            self.closed = True
            self.send_close()
            self.gaplic.add_callback(self.send_event, self.gsock, 'EV_DROP')

        if not self.on_close_broadcasted:
            self.on_close_broadcasted = True
            self.gaplic.add_callback(self.broadcast_event, 'EV_ON_CLOSE')

        self.set_new_state('ST_WAITING_HANDSHAKE')

    def frame_completed(self):
        """ Process the completed frame.
        """
        cur_frame = self.cur_frame
        gmsg = self.frame_gmsg
        gmsg.reset_rd()

        unmasked = None
        if cur_frame.frame_length:
            ln = gmsg.bytesleft()
            unmasked = gmsg.getdata(ln)
            if cur_frame.h_mask:
                h_mask = cur_frame.masking_key
                for i in list(range(ln)):
                    unmasked[i] = unmasked[i] ^ h_mask[i % 4]

        if cur_frame.h_fin:
            # last frame of message
            if self.message_buffer:
                self.message_buffer.append(unmasked)
                message = self.message_buffer.get()
                operation = self.message_header.h_opcode
            else:
                message = unmasked
                operation = cur_frame.h_opcode

            if operation == OPCODE_CONTINUATION_FRAME:
                self.logger.error("%r: Websocket CONTINUATION FRAME" % (self))
            elif operation == OPCODE_TEXT_FRAME:
                if message:
                    try:
                        message = message.decode("utf-8")
                    except:
                        self.logger.error("ERROR websocket decoding utf-8")
                        self.logger.error(hexdump('<==', message))

                    if self.trace_mach:
                        self.logger.info(
                            "%r: RECEIVE Websocket text FRAME" % (self)
                        )
                        self.logger.info(hexdump('<==', message))

                    self.gaplic.add_callback(
                        self.broadcast_event,
                        'EV_ON_MESSAGE',
                        data=message
                    )
            elif operation == OPCODE_BINARY_FRAME:
                if message:
                    if self.trace_mach:
                        self.logger.debug(
                            "%r: RECEIVE Websocket binary FRAME: %r" % (
                            self, message)
                        )
                    self.gaplic.add_callback(
                        self.broadcast_event,
                        'EV_ON_MESSAGE',
                        data=unmasked
                    )
            elif operation == OPCODE_CONTROL_CLOSE:
                self.close()
            elif operation == OPCODE_CONTROL_PING:
                if self.trace_mach:
                    self.logger.debug("%r: RECEIVE Websocket Control PING" % (
                        self)
                    )
                self.pong()
            elif operation == OPCODE_CONTROL_PONG:
                if self.trace_mach:
                    self.logger.debug("%r: RECEIVE Websocket Control PONG" % (
                        self)
                    )
            else:
                self.logger.error("ERROR %r: Websocket BAD OPCODE %r" % (
                    self, operation)
                )

            self.message_header = None
            self.message_buffer = None

        else:
            # Message with several frames
            if not self.message_header:
                self.message_header = cur_frame
            if not self.message_buffer:
                self.message_buffer = OverflowableBuffer(
                    self.config.outbuf_overflow)
            self.message_buffer.append(unmasked)

        gmsg.reset_wr()  # Reset buffer for next frame
        cur_frame.busy = False

    def ping(self):
        self._write_frame(True, OPCODE_CONTROL_PING, b'')

    def pong(self):
        self._write_frame(True, OPCODE_CONTROL_PONG, b'')

    def send_close(self, status=STATUS_NORMAL, reason=b''):
        if status < 0 or status > 0xFFFF:
            raise ValueError("code is invalid range")
        self._write_frame(
            True,
            OPCODE_CONTROL_CLOSE,
            STRUCT_2BYTES.pack(status) + bytes_(reason)
        )

    def send(self, message, binary=False):
        """ Send message to the client.
            Api websocket.
        """
        if isinstance(message, dict):
            message = ginsfsm.escape.json_encode(message)

        self.write_message(message, binary)

    def send_message(self, msg):
        """ Send jsonfied message compatible with sockjs.

        Compatible with sockjs protocol.
        """
        msg = ginsfsm.escape.json_encode(msg)
        self.write_message('a[%s]' % msg)

    def send_jsonfied_message(self, msg):
        """ Send jsonfied message. Not sockjs compatible.
        """
        msg = ginsfsm.escape.json_encode(msg)
        self.write_message(msg)

    def write_message(self, message, binary=False):
        """Sends the given message to the client of this Web Socket."""
        if binary:
            h_opcode = 0x2
        else:
            h_opcode = 0x1
        message = bytes_(message, "utf-8")
        self._write_frame(True, h_opcode, message)

    def _write_frame(self, h_fin, h_opcode, data):
        if h_fin:
            byte1 = h_opcode | 0x80
        else:
            byte1 = h_opcode

        ln = len(data)

        if ln < 126:
            byte2 = ln
            if not self.config.iam_server:
                byte2 |= 0x80
            frame = STRUCT_BB.pack(byte1, byte2)
        elif ln <= 0xFFFF:
            byte2 = 126
            if not self.config.iam_server:
                byte2 |= 0x80
            frame = STRUCT_BBH.pack(byte1, byte2, ln)

        elif ln <= 0xFFFFFFFF:
            byte2 = 127
            if not self.config.iam_server:
                byte2 |= 0x80
            frame = STRUCT_BBQ.pack(byte1, byte2, ln)

        else:
            raise ValueError("data is too long")

        if not self.config.iam_server:
            mask_key = urandom(4)
            #s = self.mask(mask_key, data)
            #data = mask_key + b"".join(s)
            data = mask_key + self.mask(mask_key, data)

        frame += data
        self.send_event(self.gsock, 'EV_SEND_DATA', data=frame)

    def mask(self, mask_key, data):
        """
        mask or unmask data. Just do xor for each byte

        mask_key: 4 byte string(byte).

        data: data to mask/unmask.
        """
        _m = array.array("B", mask_key)
        _d = array.array("B", data)
        for i in list(range(len(_d))):
            _d[i] ^= _m[i % 4]
        return _d.tostring()

    def _process_request(self, data):
        new_request = None
        cur_request = self.parsing_request
        while data:
            if cur_request is None:
                cur_request = HTTPRequestParser()
            n = cur_request.received(data)

            if cur_request.completed:
                # The cur_request (with the body) is ready to use.
                self.parsing_request = None
                new_request = cur_request
                cur_request = None
            else:
                self.parsing_request = cur_request
            if n >= len(data):
                break
            data = data[n:]

        return new_request

    def _process_response(self, data):
        new_response = None
        cur_response = self.parsing_response
        while data:
            if cur_response is None:
                cur_response = HTTPResponseParser()
            n = cur_response.received(data)

            if cur_response.completed:
                self.parsing_response = None
                new_response = cur_response
                cur_response = None
            else:
                self.parsing_response = cur_response
            if n >= len(data):
                break
            data = data[n:]

        return new_response



# -*- coding: utf-8 -*-
"""
Sphinx extension that displays the API documentation:
    * a table with the :ref:`gobj` parameters.
    * a svg image representing the :ref:`smachine`.
"""
import posixpath
import os
from os import path
import re
import imp
from math import ceil

import docutils
from docutils import nodes
from docutils.parsers.rst import Directive, directives

from ginsfsm.fsmdraw.fsm2image import fsm2image
from ginsfsm.compat import iteritems_


class smachinenode(nodes.General, nodes.Element):
    pass


class GinsFSMDirective(Directive):
    """
    Directive to insert arbitrary dot markup.
    """
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {
        'fsm': directives.unchanged,
        'gconfig': directives.unchanged,
    }
    domain = 'py'

    def run(self):
        fsm_node = smachinenode()
        fsm = self.options['fsm']  # required
        fsm_node['fsm'] = fsm

        gconfig_node = []
        gconfig = self.options.get('gconfig')  # optional
        if gconfig:
            gconfig_node = [build_gconfig_node(self, gconfig)]
        return gconfig_node + [fsm_node]


def rst2node(data):
    """Converts a reStructuredText into its node
    """
    if not data:
        return
    parser = docutils.parsers.rst.Parser()
    document = docutils.utils.new_document('<>')
    document.settings = docutils.frontend.OptionParser().get_default_values()
    document.settings.tab_width = 4
    document.settings.pep_references = False
    document.settings.rfc_references = False
    parser.parse(data, document)
    if len(document.children) == 1:
        return document.children[0]
    else:
        par = docutils.nodes.paragraph()
        for child in document.children:
            par += child
        return par


def gene_paragraph(rawtext):
    """ Generate paragraph
    """
    paragraph = nodes.paragraph()
    paragraph += nodes.Text(rawtext)
    return paragraph


def build_table(table_data, col_widths, col_rest,
                header_rows=1, stub_columns=0):
    """ build table
    """
    table = nodes.table()
    table.attributes['border'] = 1
    tgroup = nodes.tgroup(cols=len(col_widths))
    table += tgroup
    for col_width in col_widths:
        colspec = nodes.colspec(colwidth=col_width)
        if stub_columns:
            colspec.attributes['stub'] = 1
            stub_columns -= 1
        tgroup += colspec
    rows = []
    for row in table_data:
        row_node = nodes.row()
        for idx, cell in enumerate(row):
            entry = nodes.entry()
            if col_rest[idx]:
                n = rst2node(cell)
            else:
                n = None
            if isinstance(n, nodes.paragraph):
                entry += n
            else:
                entry += gene_paragraph(cell)
            row_node += entry
        rows.append(row_node)
    if header_rows:
        thead = nodes.thead()
        thead.extend(rows[:header_rows])
        tgroup += thead
    tbody = nodes.tbody()
    tbody.extend(rows[header_rows:])
    tgroup += tbody
    return table


def build_gconfig_node(self, gconfig_name):
    src = self.state.document.current_source.split(':')[0]
    gconfig_id = self.state.document.current_source.split(':')[1]
    gconfig_id = gconfig_id.split()[2]
    gconfig_id = gconfig_id + '.gconfig'
    module, ext = os.path.splitext(os.path.split(src)[-1])
    #TODO load_source is deprecated in python3.3.
    # It must be changed by load_module
    py_mod = imp.load_source(module, src)
    gconfig = getattr(py_mod, gconfig_name)

    gconfig_node = nodes.definition_list('', ids=[gconfig_id])

    rows = []
    rows.append(['Name', 'Type', 'Default value', 'Description'])
    for parameter_name, definition in iteritems_(gconfig):
        type_, default_value, flag, validate_function, desc = definition
        if type_ is None:
            type_ = 'None'
        else:
            type_ = type_.__name__
        type_ = "``%s``" % type_
        parameter_name = "``%s``" % parameter_name
        default_value = str(default_value)
        row = [parameter_name, type_, default_value, desc]
        rows.append(row)

    cols_len = [40, 20, 30, 100]
    cols_rest = [1, 1, 0, 1]
    table = build_table(rows, cols_len, cols_rest)

    title = "Configurable Parameters:"
    dl_item = nodes.definition_list_item(
        "",
        nodes.term("", "", nodes.emphasis(title, title)),
        nodes.definition("", table)
    )
    gconfig_node += dl_item

    return gconfig_node


def render_smachine(self, node, fsm_name, output_format):
    src = node.source.split(':')[0]
    module, ext = os.path.splitext(os.path.split(src)[-1])
    try:
        py_mod = imp.load_source(module, src)
    except:
        print('ERROR importing "%s" module from %s' % (module, src))
        raise
    fsm = getattr(py_mod, fsm_name)

    fname = '%s.%s' % (fsm_name, output_format)

    if hasattr(self.builder, 'imgpath'):
        # HTML
        relfn = posixpath.join(self.builder.imgpath, fname)
        outfn = path.join(self.builder.outdir, '_images', fname)
        imgdir = path.join(self.builder.outdir, '_images')
        try:
            os.makedirs(imgdir)
        except OSError:
            pass

    else:
        # LaTeX
        relfn = fname
        outfn = path.join(self.builder.outdir, fname)

    # TODO add title? fsm_node += nodes.title(text='Diagram:\n')

    # TODO: create and use the maxwidth option
    fsm2image(outfn, fsm, fsm_name, output_format)

    return relfn, outfn


svg_dim_re = re.compile(r'<svg\swidth="(\d+)pt"\sheight="(\d+)pt"', re.M)


def get_svg_tag(svgref, svgfile):
    # Webkit can't figure out svg dimensions when using object tag
    # so we need to get it from the svg file
    fp = open(svgfile, 'r')
    try:
        for line in fp:
            match = svg_dim_re.match(line)
            if match:
                dimensions = match.groups()
                break
        else:
            dimensions = None
    finally:
        fp.close()

    # We need this hack to make WebKit show our object tag properly
    def pt2px(x):
        return int(ceil((96.0 / 72.0) * float(x)))

    if dimensions:
        style = ' width="%s" height="%s"' % tuple(list(map(pt2px, dimensions)))
    else:
        style = ''

    # The object tag works fine on Firefox and WebKit
    # Besides it's a hack, this strategy does not mess with templates.
    #return '<object type="image/svg+xml" data="%s"%s></object>\n' % \
    #       (svgref, style)
    return '<embed type="image/svg+xml" src="%s"%s/>\n' % \
           (svgref, style)


def render_smachine_html(self, node, smachine):
    fname, outfn = render_smachine(self, node, smachine, 'svg')

    self.body.append(self.starttag(node, 'dl', CLASS='smachine'))
    self.body.append('<dt><em>Diagram:</em></dt>')

    self.body.append('<dd>')
    svgtag = get_svg_tag(fname, outfn)
    self.body.append(svgtag)
    self.body.append('</dd>')

    self.body.append('</dl>\n')
    raise nodes.SkipNode


def html_visit_smachinenode(self, node):
    render_smachine_html(self, node, node['fsm'])


def render_smachine_latex(self, node, smachine):
    fname, outfn = render_smachine(self, smachine, 'pdf')
    inline = node.get('inline', False)
    if inline:
        para_separator = ''
    else:
        para_separator = '\n'

    if fname is not None:
        caption = node.get('caption')
        # XXX add ids from previous target node
        if caption and not inline:
            self.body.append('\n\\begin{figure}[h!]')
            self.body.append('\n\\begin{center}')
            self.body.append('\n\\caption{%s}' % self.encode(caption))
            self.body.append('\n\\includegraphics{%s}' % fname)
            self.body.append('\n\\end{center}')
            self.body.append('\n\\end{figure}\n')
        else:
            self.body.append('%s\\includegraphics{%s}%s' %
                             (para_separator, fname, para_separator))
    raise nodes.SkipNode


def latex_visit_smachinenode(self, node):
    render_smachine_latex(self, node, node['fsm'])


def render_smachine_texinfo(self, node, smachine):
    fname, outfn = render_smachine(self, smachine, 'png')
    if fname is not None:
        self.body.append('\n\n@float\n')
        caption = node.get('caption')
        if caption:
            self.body.append('@caption{%s}\n' % self.escape_arg(caption))
        self.body.append('@image{%s,,,[smachinenode],png}\n'
                         '@end float\n\n' % fname[:-4])
    raise nodes.SkipNode


def texinfo_visit_smachinenode(self, node):
    render_smachine_texinfo(self, node, node['fsm'])


def text_visit_smachinenode(self, node):
    self.add_text('[graph]')


def man_visit_smachinenode(self, node):
    self.body.append('[graph]')
    raise nodes.SkipNode


def setup(app):
    """Sphinx setup."""
    app.add_node(smachinenode,
                 html=(html_visit_smachinenode, None),
                 latex=(latex_visit_smachinenode, None),
                 texinfo=(texinfo_visit_smachinenode, None),
                 text=(text_visit_smachinenode, None),
                 man=(man_visit_smachinenode, None))
    app.add_directive('ginsfsm', GinsFSMDirective)
